@@ -1,7 +1,7 @@
 /*
  * tools/e2fsadm.c
  *
- * Copyright (C) 1997 - 2000  Heinz Mauelshagen, Germany
+ * Copyright (C) 1997 - 2000  Heinz Mauelshagen, Sistina Software
  *
  * June,September 1998
  * February,July,October 1999
@@ -39,14 +39,10 @@
  *    01/01/2000 - take care of rounding if striped resize
  *                 (idea by Pete Allen <petea3@attglobal.net>)
  *    15/02/2000 - use lvm_error()
- *    17/07/2000 - Add support for ext2online <adilger@turbolinux.com>
- *    18/07/2000 - use getmntent on /etc/mtab, then /proc/mounts for mounted LV
- *               - also check mountpoint in addition to device name.
  *
  */
 
 #include <lvm_user.h>
-#include <mntent.h>
 
 char *cmd = NULL;
 
@@ -65,7 +61,6 @@ typedef struct {
 } ext2_super_t;
 
 static char *bin_dirs[] = { "/sbin", "/usr/sbin", "/usr/local/sbin", "/bin", "/usr/bin", "/usr/local/bin", NULL};
-static char *mnt_files[] = { MOUNTED, "/proc/mounts", MNTTAB , NULL};
 
 /* internal function prototypes */
 char *lvm_find_command ( char *);
@@ -77,6 +72,7 @@ int main ( int argc, char **argv) {
    int c = 0;
    int grow = UNDEF;
    int l = 0;
+   int len = 0;
    int opt_l = 0;
    int opt_L = 0;
    int opt_n = 0;
@@ -90,24 +86,21 @@ int main ( int argc, char **argv) {
    long size = 0;
    char *cmd_fsck = "e2fsck";
    char *cmd_resize = NULL;
-   char *cmd_resize_opts = NULL;
    char *cmd_resize_default = "resize2fs";
+   char *cmd_resize_opts = NULL;
    char *cmd_resize_opts_default = "-p";
-   char *cmd_resize_default2 = "ext2resize";
-   char *cmd_resize_opts_default2 = "-v";
-   char *cmd_online = NULL;
-   char *cmd_online_default = "ext2online";
-   char *cmd_online_opts = NULL;
-   char *cmd_online_opts_default = "-v";
    char *cmd_extend = "lvextend";
    char *cmd_ptr = NULL;
    char *cmd_reduce = "lvreduce";
    char *dummy = NULL;
    char *lv_name = NULL;
+   char *mount = NULL;
    char *vg_name = NULL;
-   char **mntfile = NULL;
+   char *proc_mounts = "/proc/mounts";
+   char *ptr = NULL;
    char *size_ptr = NULL;
-   char command[2*NAME_LEN];
+   char buffer[2*NAME_LEN] = { 0, };
+   char command[2*NAME_LEN] = { 0, };
 #ifdef DEBUG
    char *options = "dh?l:L:ntv";
 #else
@@ -125,6 +118,7 @@ int main ( int argc, char **argv) {
       { "verbose", no_argument,       NULL, 'v'},
       { NULL,      0,                 NULL, 0}
    };
+   FILE *proc = NULL;
    lv_t *lv = NULL;
    vg_t *vg = NULL;
    ext2_super_t *super = NULL;
@@ -221,28 +215,16 @@ int main ( int argc, char **argv) {
 
    CMD_MINUS_CHK;
 
-   if ( ( cmd_resize = getenv ( "E2FSADM_RESIZE_CMD" )) == NULL) {
-      if ( lvm_find_command ( cmd_resize_default)) {
-         cmd_resize = cmd_resize_default;
-         cmd_resize_opts = cmd_resize_opts_default;
-      } else if ( lvm_find_command ( cmd_resize_default2)) {
-         cmd_resize = cmd_resize_default2;
-         cmd_resize_opts = cmd_resize_opts_default2;
-      }
+   if ( ( cmd_resize = getenv ( "E2FSADM_RESIZE_CMD" )) == NULL)
+      cmd_resize = cmd_resize_default;
+   if ( ( cmd_resize_opts = getenv ( "E2FSADM_RESIZE_OPTS" )) == NULL)
+      cmd_resize_opts = cmd_resize_opts_default;
+
+   if ( lvm_find_command ( cmd_resize) == NULL) {
+      lvm_path_error ( cmd_resize, "resize");
+      return LVM_EE2FSADM_RESIZE_PATH;
    }
 
-   if ( ( dummy = getenv ( "E2FSADM_RESIZE_OPTS" )))
-      cmd_online_opts = dummy;
-
-   if ( ( cmd_online = getenv ( "E2FSADM_ONLINE_CMD" )) == NULL) {
-      cmd_online = cmd_online_default;
-      cmd_online_opts = cmd_online_opts_default;
-   }
-
-   if ( ( dummy = getenv ( "E2FSADM_ONLINE_OPTS" )))
-      cmd_online_opts = dummy;
-
-
    if ( opt_l == 0 && opt_L == 0) {
       fprintf ( stderr, "%s -- please give either an l or an L option\n\n",
                         cmd);
@@ -255,47 +237,10 @@ int main ( int argc, char **argv) {
       return LVM_EE2FSADM_LV_MISSING;
    }
 
-   if ( opt_t > 0) printf ( "%s -- this is a test run and no real resize\n",
-                            cmd);
-
    lv_name = argv[optind];
 
-   /* check to see if the LV or mountpoint appears in one of the mtab files */
-   for (mntfile = mnt_files; *mntfile != NULL; mntfile++) {
-      struct mntent *mntent;
-      FILE *mounts;
-
-      if ( ( mounts = setmntent ( *mntfile, "r")) == NULL)
-         continue;
-
-      while ( (mntent = getmntent ( mounts)) != NULL) {
-         /* Also check the mount point, and update lv_name if it matches */
-         if (strcmp (lv_name, mntent->mnt_dir) == 0)
-            lv_name = mntent->mnt_fsname;
-         if ( strcmp ( lv_name, mntent->mnt_fsname) == 0) {
-            /* If we aren't looking at /etc/fstab, the filesystem is mounted */
-            if (strcmp(*mntfile, MNTTAB)) {
-               if ( lvm_find_command ( cmd_online) == NULL) {
-                  lvm_path_error ( cmd_online, "online resize");
-                  fprintf ( stderr, "%s -- ERROR: '%s' is mounted on '%s'.\n"
-                                    "%s -- please umount it to resize\n\n",
-                                    cmd, lv_name, mntent->mnt_dir, cmd);
-                  endmntent(mounts);
-                  return LVM_EE2FSADM_RESIZE_PATH;
-               }
-               /* FIXME we could check the mnt_type for reiserfs here and
-                *       set cmd_resize to resize_reiserfs and its options.
-                */
-               cmd_resize = cmd_online;
-               cmd_resize_opts = cmd_online_opts;
-            }
-            /* don't close mounts, in case we point to mnt_fsname */
-            goto have_mntent;
-         }
-      }
-      endmntent(mounts);
-   }
-have_mntent:
+   if ( opt_t > 0) printf ( "%s -- this is a test run and no real resize\n",
+                            cmd);
 
    if ( opt_v > 0) printf ( "%s -- checking logical volume name\n", cmd);
    if ( lv_check_name ( lv_name) < 0) {
@@ -398,6 +343,30 @@ have_mntent:
       printf ( "%ld KB\n", size / 2);
    }
 
+   if ( ( proc = fopen ( proc_mounts, "r")) == NULL) {
+      fprintf ( stderr, "%s -- ERROR: can't open \"%s\"\n\n",
+                        cmd, proc_mounts);
+      return LVM_EE2FSADM_PROC_MOUNTS;
+   }
+
+   len = strlen ( lv_name);
+   while ( fgets ( buffer, sizeof ( buffer), proc) != NULL) {
+      if ( strncmp ( lv_name, buffer, len) == 0) {
+         mount = buffer;
+         while ( *mount != ' ') mount++;
+         mount++;
+         ptr = mount;
+         while ( *ptr != ' ') ptr++;
+         *ptr = 0;
+         fprintf ( stderr, "%s -- ERROR: \"%s\" is mounted on \"%s\"\n"
+                           "%s -- please umount it to resize\n\n",
+                           cmd, lv_name, mount, cmd);
+         fclose ( proc);
+         return LVM_EE2FSADM_MOUNTED;
+      }
+   }
+   fclose ( proc);
+
    if ( ( super = lvm_is_ext2_fs ( lv_name)) == NULL)
       return LVM_EE2FSADM_NO_EXT2;
 
@@ -407,7 +376,7 @@ have_mntent:
       return LVM_EE2FSADM_LV_SIZE;
    }
 
-   if ( opt_n == 0 && cmd_resize != cmd_online) {
+   if ( opt_n == 0) {
       if ( opt_v > 0) printf ( "%s -- checking filesystem (this can "
                                "take a while)\n", cmd);
       if ( ( cmd_ptr = lvm_find_command ( cmd_fsck)) == NULL) {
@@ -415,7 +384,7 @@ have_mntent:
          return LVM_EE2FSADM_FSCK_PATH;
       }
       if ( opt_t == 0) {
-         sprintf ( command, "%s -f %s", cmd_ptr, lv_name);
+         sprintf ( command, "%s -f %s%c", cmd_ptr, lv_name, 0);
          if ( system ( command) != 0) return  LVM_EE2FSADM_FSCK_RUN;
       }
    }
@@ -427,8 +396,8 @@ have_mntent:
          lvm_path_error ( cmd_extend, "extend logical volume");
          return LVM_EE2FSADM_LV_EXTEND_PATH;
       }
-      sprintf ( command, "%s -l %lu %s",
-                         cmd_ptr, size / vg->pe_size, lv_name);
+      sprintf ( command, "%s -l %lu %s%c",
+                         cmd_ptr, size / vg->pe_size, lv_name, 0);
       if ( opt_t == 0) {
          if ( system ( command) != 0) return LVM_EE2FSADM_LV_EXTEND_RUN;
          if ( ( ret = lvm_tab_lv_read_by_name ( vg_name, lv_name, &lv)) < 0) {
@@ -446,13 +415,13 @@ have_mntent:
       }
       if ( opt_v > 0) printf ( "%s -- resizing filesystem (this can "
                                "take a while)\n", cmd);
-      if ( !cmd_resize || ( cmd_ptr = lvm_find_command ( cmd_resize)) == NULL) {
-         lvm_path_error(cmd_resize ? cmd_resize : cmd_resize_default, "resize");
+      if ( ( cmd_ptr = lvm_find_command ( cmd_resize)) == NULL) {
+         lvm_path_error ( cmd_resize, "resize");
          return LVM_EE2FSADM_RESIZE_PATH;
       }
-      sprintf ( command, "%s %s %s %lu",
+      sprintf ( command, "%s %s %s %lu%c",
                          cmd_ptr, cmd_resize_opts,
-                         lv_name, ( size / 2)>>super->block_size);
+                         lv_name, ( size / 2)>>super->block_size, 0);
       if ( opt_t == 0) {
          if ( system ( command) != 0) return LVM_EE2FSADM_RESIZE_RUN;
          if ( ( super = lvm_is_ext2_fs ( lv_name)) == NULL)
@@ -468,13 +437,13 @@ have_mntent:
    } else { /* grow == FALSE */
       if ( opt_v > 0) printf ( "%s -- resizing filesystem (this can "
                                "take a while)\n", cmd);
-      if ( !cmd_resize || ( cmd_ptr = lvm_find_command ( cmd_resize)) == NULL) {
-         lvm_path_error(cmd_resize ? cmd_resize : cmd_resize_default, "resize");
+      if ( ( cmd_ptr = lvm_find_command ( cmd_resize)) == NULL) {
+         lvm_path_error ( cmd_resize, "resize");
          return LVM_EE2FSADM_RESIZE_PATH;
       }
-      sprintf ( command, "%s %s %s %lu",
+      sprintf ( command, "%s %s %s %lu%c",
                          cmd_ptr, cmd_resize_opts,
-                         lv_name, ( size / 2)>>super->block_size);
+                         lv_name, ( size / 2)>>super->block_size, 0);
       if ( opt_t == 0) {
          if ( system ( command) != 0) return LVM_EE2FSADM_RESIZE_RUN;
          if ( ( super = lvm_is_ext2_fs ( lv_name)) == NULL)
@@ -491,8 +460,8 @@ have_mntent:
          lvm_path_error ( cmd_reduce, "reduce logical volume");
          return LVM_EE2FSADM_LV_REDUCE_PATH;
       }
-      sprintf ( command, "%s -f -l %lu %s",
-                         cmd_ptr, size / vg->pe_size, lv_name);
+      sprintf ( command, "%s -f -l %lu %s%c",
+                         cmd_ptr, size / vg->pe_size, lv_name, 0);
       if ( opt_t == 0) {
          if ( system ( command) != 0) return LVM_EE2FSADM_LV_REDUCE_RUN;
       }
@@ -515,7 +484,7 @@ char *lvm_find_command ( char *cmd_to_find) {
    if ( cmd_to_find == NULL || strlen ( cmd_to_find) == 0) return NULL;
 
    while ( bin_dirs[i] != NULL) {
-      sprintf ( command, "%s/%s", bin_dirs[i], cmd_to_find);
+      sprintf ( command, "%s/%s%c", bin_dirs[i], cmd_to_find, 0);
       if ( stat ( command, &stat_b) == 0) return command;
       i++;
    }
