@@ -1,10 +1,10 @@
 /*
  * tools/lib/lvm_check_partitioned_dev.c
  *
- * Copyright (C) 1999  Heinz Mauelshagen, Germany
+ * Copyright (C) 1999  Heinz Mauelshagen, Sistina Software
  *
  * March,July,August 1999
- * January 2000
+ * January,April 2000
  *
  *
  * This LVM library is free software; you can redistribute it and/or
@@ -30,6 +30,7 @@
  *    05/07/1999 - added 2.3.x new IDE majors
  *    20/08/1999 - added DAC960 majors
  *    31/01/2000 - use debug_enter()/debug_leave()
+ *    26/04/2000 - added DASD major support
  *
  */
 
@@ -56,6 +57,8 @@ dev_t valid_majors[] = {
    IDE3_MAJOR,
 #ifdef IDE4_MAJOR
    IDE4_MAJOR,
+#endif
+#ifdef IDE5_MAJOR
    IDE5_MAJOR,
 #endif
 #ifdef IDE6_MAJOR
@@ -73,6 +76,9 @@ dev_t valid_majors[] = {
    DAC960_MAJOR+5,
    DAC960_MAJOR+6,
    DAC960_MAJOR+7,
+#endif
+#ifdef	DASD_MAJOR
+   DASD_MAJOR,
 #endif
    -1
 };
@@ -144,17 +150,24 @@ int lvm_partition_count ( dev_t st_rdev) {
 #endif
    if ( st_rdev == 0) ret = -LVM_EPARAM;
    else {
-      for ( i = 0; valid_majors[i] != -1; i++) {
-         if ( MAJOR ( st_rdev) == valid_majors[i]) {
-            for ( j = 0; valid_DAC960_majors[j] != -1; j++) {
-               if ( valid_majors[i] == valid_DAC960_majors[j]) {
-                  ret = 8;
-                  break;
+#ifdef DASD_MAJOR
+      if ( MAJOR ( st_rdev) == DASD_MAJOR) ret = 4;
+      else {
+#endif
+         for ( i = 0; valid_majors[i] != -1; i++) {
+            if ( MAJOR ( st_rdev) == valid_majors[i]) {
+               for ( j = 0; valid_DAC960_majors[j] != -1; j++) {
+                  if ( valid_majors[i] == valid_DAC960_majors[j]) {
+                     ret = 8;
+                     break;
+                  }
                }
+               break;
             }
-            break;
          }
+#ifdef DASD_MAJOR
       }
+#endif
    }
 
 #ifdef DEBUG
