@@ -1,9 +1,9 @@
 /*
  * tools/lib/lvm_error.c
  *
- * Copyright (C)  2000  Heinz Mauelshagen, Germany
+ * Copyright (C)  2000  Heinz Mauelshagen, Sistina Software
  *
- * February 2000
+ * February,March,July,October 2000
  *
  * LVM is free software; you can redistribute it and/or modify
  * it under the terms of the GNU General Public License as published by
@@ -22,13 +22,21 @@
  *
  */
 
+/*
+ * Changelog
+ *
+ *   17/03/2000 - changed namelist to uuidlist
+ *   01/07/2000 - extended snaphsot related data
+ *   30/10/2000 - use strerror() in I am not apple to supply one of my own
+ *
+ */
+
 #include <liblvm.h>
 
 
 char *lvm_error ( int error) {
    int c = 0;
    char *ret = NULL;
-   static char buffer[16];
    struct lvm_error_struct {
       ushort error;
       char *errstr;
@@ -77,35 +85,47 @@ char *lvm_error ( int error) {
       { LVM_ELV_CHECK_NAME_LV_NUM, "lv_check_name(): LV number"},
       { LVM_ELV_CHECK_NAME_VG_NAME, "lv_check_name(): VG name"},
       { LVM_ELV_CHECK_STRIPE_SIZE, "lv_check_stripe_size()"},
-      { LVM_ELV_CREATE_MODE_CHMOD, "lv_create_node(): chmod"},
-      { LVM_ELV_CREATE_MODE_MKNOD,  "lv_create_node(): mknod"},
+      { LVM_ELV_CREATE_NODE_CHMOD, "lv_create_node(): chmod"},
+      { LVM_ELV_CREATE_NODE_MKNOD,  "lv_create_node(): mknod"},
       { LVM_ELV_CREATE_NODE_UNLINK,  "lv_create_node(): unlink"},
       { LVM_ELV_CREATE_REMOVE_OPEN,  "lv_create_remove(): open"},
       { LVM_ELV_CURRENT_LE, "lv_check_consistency(): current LE"},
       { LVM_ELV_EXTEND_REDUCE_OPEN, "lv_extend_reduce(): open"},
       { LVM_ELV_LE_REMAP_OPEN, "lv_le_remap(): open"},
+      { LVM_ELV_INIT_COW_TABLE_MALLOC, "lv_init_COW_table(): malloc"},
+      { LVM_ELV_INIT_COW_TABLE_CLOSE, "lv_init_COW_table(): close"},
+      { LVM_ELV_INIT_COW_TABLE_OPEN, "lv_init_COW_table(): open"},
+      { LVM_ELV_INIT_COW_TABLE_LLSEEK, "lv_init_COW_table(): llseek"},
+      { LVM_ELV_INIT_COW_TABLE_WRITE, "lv_init_COW_table(): write"},
+      { LVM_ELV_LE_REMAP_OPEN, "lv_le_remap(): open"},
       { LVM_ELV_LVNAME, "lv_check_consistency(): LV name"},
       { LVM_ELV_MIRROR_COPIES, "lv_check_consistency(): LV mirror copies"},
       { LVM_ELV_NUMBER, "lv_check_consistency(): LV number"},
       { LVM_ELV_OPEN, "lv_check_consistency(): LV # open"},
-      { LVM_ELV_READ_ALL_LV_OF_VG_LSEEK, "lv_read_all_lv_of_vg(): lseek"},
-      { LVM_ELV_READ_ALL_LV_OF_VG_MALLOC, "lv_read_all_lv_of_vg(): malloc"},
-      { LVM_ELV_READ_ALL_LV_OF_VG_NL, "lv_read_all_lv_of_vg(): number of LV"},
-      { LVM_ELV_READ_ALL_LV_OF_VG_OPEN, "lv_read_all_lv_of_vg(): open"},
-      { LVM_ELV_READ_ALL_LV_OF_VG_READ, "lv_read_all_lv_of_vg(): read"},
-      { LVM_ELV_READ_ALL_LV_OF_VG_VG_READ, "lv_read_all_lv_of_vg(): vg_read"},
-      { LVM_ELV_READ_BYINDEX_LV_READ_ALL_LV_OF_VG,
-        "lv_read_byindex(): lv_read_all_lv_of_vg"},
+      { LVM_ELV_READ_ALL_LV_LSEEK, "lv_read_all_lv(): lseek"},
+      { LVM_ELV_READ_ALL_LV_MALLOC, "lv_read_all_lv(): malloc"},
+      { LVM_ELV_READ_ALL_LV_NL, "lv_read_all_lv(): number of LV"},
+      { LVM_ELV_READ_ALL_LV_OPEN, "lv_read_all_lv(): open"},
+      { LVM_ELV_READ_ALL_LV_READ, "lv_read_all_lv(): read"},
+      { LVM_ELV_READ_ALL_LV_VG_READ, "lv_read_all_lv(): vg_read"},
+      { LVM_ELV_READ_COW_TABLE_MALLOC, "lv_read_COW_table(): malloc"},
+      { LVM_ELV_READ_COW_TABLE_OPEN, "lv_read_COW_table(): open"},
+      { LVM_ELV_READ_COW_TABLE_LLSEEK, "lv_read_COW_table(): llseek"},
+      { LVM_ELV_READ_COW_TABLE_READ, "lv_read_COW_table(): read"},
+      { LVM_ELV_READ_COW_TABLE_CLOSE, "lv_read_COW_table(): close"},
+      { LVM_ELV_READ_BYINDEX_LV_READ_ALL_LV,
+        "lv_read_byindex(): lv_read_all_lv"},
       { LVM_ELV_READ_BYINDEX_VG_READ, "lv_read_byindex(): vg_read"},
       { LVM_ELV_READ_BYINDEX_VG_NAME, "lv_read_byindex(): vg_name"},
       { LVM_ELV_READ_LV, "lv_read(): read LV"},
       { LVM_ELV_READ_LV_NAME, "lv_read(): LV name"},
-      { LVM_ELV_READ_LV_READ_ALL_LV_OF_VG,
-        "lv_read_all_lv_of_vg(): lv_read_all_lv_of_vg"},
+      { LVM_ELV_READ_LV_READ_ALL_LV,
+        "lv_read_all_lv(): lv_read_all_lv"},
       { LVM_ELV_READ_VG_READ, "lv_read(): vg_read"},
       { LVM_ELV_READ_VG_NAME, "lv_read(): VG name"},
       { LVM_ELV_RECOVERY, "lv_check_consistency(): LV recovery"},
       { LVM_ELV_RELEASE_LV_NUM, "lv_release(): LV number"},
+      { LVM_ELV_RENAME_OPEN, "lv_rename(): PV open"},
       { LVM_ELV_SCHEDULE, "lv_check_consistency(): LV schedule"},
       { LVM_ELV_SETUP_FOR_CREATE_LVM_TAB_GET_FREE_BLK_DEV,
         "lv_setup_for_create(): lvm_tab_get_free_blk_dev"},
@@ -120,8 +140,8 @@ char *lvm_error ( int error) {
       { LVM_ELV_SETUP_FOR_EXTEND_STRIPES, "lv_setup_for_extend(): LV index"},
       { LVM_ELV_SETUP_FOR_REDUCE_LV_INDEX, "lv_setup_for_reduce(): LV index"},
       { LVM_ELV_SETUP_FOR_REDUCE_MALLOC, "lv_setup_for_reduce(): malloc"},
-      { LVM_ELV_SETUP_SNAPSHOT_EXCEPTION_TABLE_MALLOC,
-        "lv_setup_snapshot_exception_table(): malloc"},
+      { LVM_ELV_SETUP_COW_TABLE_FOR_CREATE_MALLOC,
+        "lv_setup_COW_table_for_create(): malloc"},
       { LVM_ELV_SHOW_CURRENT_PE_TEXT_LV_INDEX,
         "lv_show_current_pe_text(): LV index"},
       { LVM_ELV_SHOW_VG_READ_WITH_PV_AND_LV,
@@ -137,10 +157,10 @@ char *lvm_error ( int error) {
       { LVM_ELV_STRIPESIZE, "lv_check_consistency(): LV stripesize"},
       { LVM_ELV_TIMEOUT, "lv_check_consistency(): LV timeout"},
       { LVM_ELV_VGNAME, "lv_check_consistency(): VG name"},
-      { LVM_ELV_WRITE_ALL_LV_OF_VG_LSEEK, "lv_write_all_lv_of_vg(): lseek"},
-      { LVM_ELV_WRITE_ALL_LV_OF_VG_MALLOC, "lv_write_all_lv_of_vg(): malloc"},
-      { LVM_ELV_WRITE_ALL_LV_OF_VG_OPEN, "lv_write_all_lv_of_vg(): open"},
-      { LVM_ELV_WRITE_ALL_LV_OF_VG_WRITE, "lv_write_all_lv_of_vg(): write"},
+      { LVM_ELV_WRITE_ALL_LV_LSEEK, "lv_write_all_lv(): lseek"},
+      { LVM_ELV_WRITE_ALL_LV_MALLOC, "lv_write_all_lv(): malloc"},
+      { LVM_ELV_WRITE_ALL_LV_OPEN, "lv_write_all_lv(): open"},
+      { LVM_ELV_WRITE_ALL_LV_WRITE, "lv_write_all_lv(): write"},
       { LVM_ELV_WRITE_LSEEK, "lv_write(): lseek"},
       { LVM_ELV_WRITE_OPEN, "lv_write(): open"},
       { LVM_ELV_WRITE_WRITE, "lv_write(): write"},
@@ -224,10 +244,10 @@ char *lvm_error ( int error) {
       { LVM_EPV_READ_LVM_STRUCT_VERSION, "pv_read(): LVM structure version"},
       { LVM_EPV_READ_MAJOR, "pv_read(): device major number"},
       { LVM_EPV_READ_MD_DEVICE, "pv_read(): multiple device"},
-      { LVM_EPV_READ_NAMELIST_LSEEK, "pv_read_namelist(): lseek"},
-      { LVM_EPV_READ_NAMELIST_MALLOC, "pv_read_namelist(): malloc"},
-      { LVM_EPV_READ_NAMELIST_OPEN, "pv_read_namelist(): open"},
-      { LVM_EPV_READ_NAMELIST_READ, "pv_read_namelist(): read"},
+      { LVM_EPV_READ_UUIDLIST_LSEEK, "pv_read_uuidlist(): lseek"},
+      { LVM_EPV_READ_UUIDLIST_MALLOC, "pv_read_uuidlist(): malloc"},
+      { LVM_EPV_READ_UUIDLIST_OPEN, "pv_read_uuidlist(): open"},
+      { LVM_EPV_READ_UUIDLIST_READ, "pv_read_uuidlist(): read"},
       { LVM_EPV_READ_OPEN, "pv_read(): open"},
       { LVM_EPV_READ_PE_LSEEK, "pv_read_pe(): lseek"},
       { LVM_EPV_READ_PE_MALLOC, "pv_read_pe(): malloc"},
@@ -254,18 +274,18 @@ char *lvm_error ( int error) {
       { LVM_EPV_STATUS_OPEN, "pv_status(): open"},
       { LVM_EPV_TIME_CHECK, "pv_time_check()"},
       { LVM_EPV_WRITE_LSEEK, "pv_write(): lseek"},
-      { LVM_EPV_WRITE_NAMELIST_LSEEK, "pv_write_namelist(): lseek"},
-      { LVM_EPV_WRITE_NAMELIST_MALLOC, "pv_write_namelist(): malloc"},
-      { LVM_EPV_WRITE_NAMELIST_OPEN, "pv_write_namelist(): open"},
-      { LVM_EPV_WRITE_NAMELIST_WRITE, "pv_write_namelist(): write"},
+      { LVM_EPV_WRITE_UUIDLIST_LSEEK, "pv_write_uuidlist(): lseek"},
+      { LVM_EPV_WRITE_UUIDLIST_MALLOC, "pv_write_uuidlist(): malloc"},
+      { LVM_EPV_WRITE_UUIDLIST_OPEN, "pv_write_uuidlist(): open"},
+      { LVM_EPV_WRITE_UUIDLIST_WRITE, "pv_write_uuidlist(): write"},
       { LVM_EPV_WRITE_OPEN, "pv_write(): open"},
       { LVM_EPV_WRITE_PE_LSEEK, "pv_write_pe(): lseek"},
       { LVM_EPV_WRITE_PE_OPEN, "pv_write_pe(): opem"},
       { LVM_EPV_WRITE_PE_SIZE, "pv_write_pe(): "},
       { LVM_EPV_WRITE_PE_WRITE, "pv_write_pe(): write"},
       { LVM_EPV_WRITE_WRITE, "pv_write(): write"},
-      { LVM_EREMOVE_RECURSIVE_MALLOC, "lvm_remouverecursive(): malloc"},
-      { LVM_EREMOVE_RECURSIVE_OPENDIR, "lvm_remouverecursive(): opendir"},
+      { LVM_EREMOVE_RECURSIVE_MALLOC, "lvm_remove_recursive(): malloc"},
+      { LVM_EREMOVE_RECURSIVE_OPENDIR, "lvm_remove_recursive(): opendir"},
       { LVM_ESIZE, "LV size error"},
       { LVM_ESYSTEM_ID_SET_UNAME, "system_id(), uname"},
       { LVM_EVG_CFGBACKUP_FILE_EXISTS, "vg_cfgbackup(): file exists"},
@@ -347,18 +367,13 @@ char *lvm_error ( int error) {
         "vg_read_with_pv_and_lv(): malloc"},
       { LVM_EVG_READ_WITH_PV_AND_LV_PV_CUR,
         "vg_read_with_pv_and_lv(): current PV"},
-      { LVM_EVG_SETUP_FOR_CREATE_MALLOC,
-        "vg_setup_for_create(): malloc"},
-      { LVM_EVG_SETUP_FOR_CREATE_PV_SIZE,
-        "vg_setup_for_create(): PV size"},
-      { LVM_EVG_SETUP_FOR_CREATE_VG_NUMBER,
-        "vg_setup_for_create(): VG number"},
-      { LVM_EVG_SETUP_FOR_EXTEND_MALLOC,
-        "vg_setup_for_extend(): malloc"},
-      { LVM_EVG_SETUP_FOR_EXTEND_MAX_PV,
-        "vg_setup_for_extend(): PV maximum"},
-      { LVM_EVG_SETUP_FOR_EXTEND_NO_PV,
-        "vg_setup_for_extend(): no PV"},
+      { LVM_EVG_RENAME_OPEN, "vg_rename(): PV open"},
+      { LVM_EVG_SETUP_FOR_CREATE_MALLOC, "vg_setup_for_create(): malloc"},
+      { LVM_EVG_SETUP_FOR_CREATE_PV_SIZE, "vg_setup_for_create(): PV size"},
+      { LVM_EVG_SETUP_FOR_CREATE_VG_NUMBER, "vg_setup_for_create(): VG number"},
+      { LVM_EVG_SETUP_FOR_EXTEND_MALLOC, "vg_setup_for_extend(): malloc"},
+      { LVM_EVG_SETUP_FOR_EXTEND_MAX_PV, "vg_setup_for_extend(): PV maximum"},
+      { LVM_EVG_SETUP_FOR_EXTEND_NO_PV, "vg_setup_for_extend(): no PV"},
       { LVM_EVG_SETUP_FOR_EXTEND_PV_ALREADY,
         "vg_setup_for_extend(): PV already recognized"},
       { LVM_EVG_SETUP_FOR_EXTEND_PV_CHECK_NAME,
@@ -367,8 +382,7 @@ char *lvm_error ( int error) {
         "vg_setup_for_extend(): pv_check_new"},
       { LVM_EVG_SETUP_FOR_EXTEND_PV_GET_SIZE,
         "vg_setup_for_extend(): pv_get_size"},
-      { LVM_EVG_SETUP_FOR_EXTEND_PV_SIZE,
-        "vg_setup_for_extend(): PV size"},
+      { LVM_EVG_SETUP_FOR_EXTEND_PV_SIZE, "vg_setup_for_extend(): PV size"},
       { LVM_EVG_SETUP_FOR_EXTEND_PV_SIZE_REL,
         "vg_setup_for_extend(): PV size too small for PE size"},
 
@@ -414,8 +428,7 @@ char *lvm_error ( int error) {
    }
 
    if ( lvm_errors[c].errstr == NULL) {
-      sprintf ( buffer, "%d%c", error, 0);
-      ret = buffer;
+      ret = strerror ( errno);
    } else ret = lvm_errors[c].errstr;
 
 #ifdef DEBUG
