@@ -1,11 +1,12 @@
 /*
  * tools/lib/pv_check_new.c
  *
- * Copyright (C) 1997 - 1999  Heinz Mauelshagen, Germany
+ * Copyright (C) 1997 - 2000  Heinz Mauelshagen, Sistina Software
  *
  * March 1997
  * May 1998
- * January 1999
+ * January,September 1999
+ * February 2000
  *
  *
  * This LVM library is free software; you can redistribute it and/or
@@ -30,6 +31,9 @@
  *
  *    12/05/1998 - avoided checking vg_name against LVM_PV_NEW
  *    13/01/1999 - enhanced checks
+ *    07/09/1999 - enhanced checks
+ *    31/01/2000 - use debug_enter()/debug_leave()
+ *    16/02/2000 - fixed zero block recognition bug
  *
  */
 
@@ -40,21 +44,21 @@ int pv_check_new ( pv_t *pv) {
    int ret = FALSE;
 
 #ifdef DEBUG
-   debug ( "pv_check_new -- CALLED\n");
+   debug_enter ( "pv_check_new -- CALLED\n");
 #endif
 
    if ( pv == NULL) ret = -LVM_EPARAM;
    else {
-      for ( i = 0; i < sizeof ( pv->vg_name); i++)
-         if ( pv->vg_name[i] != 0) 
-            break;
-   
-      if ( i == sizeof ( pv->vg_name) &&
-           pv->pe_total == 0) ret = TRUE;
+      for ( i = 0; i < sizeof ( pv->vg_name); i++) {
+         if ( pv->vg_name[i] != 0) break;
+      }
+      if ( i == sizeof ( pv->vg_name)) ret = TRUE;
+      else                             ret = FALSE;
    }
 
+
 #ifdef DEBUG
-   debug ( "pv_check_new -- LEAVING with ret: %d\n", ret);
+   debug_leave ( "pv_check_new -- LEAVING with ret: %d\n", ret);
 #endif
    return ret;
 }
