@@ -1,10 +1,11 @@
 /*
  * tools/lib/pv_check_part.c
  *
- * Copyright (C) 1997 - 1999  Heinz Mauelshagen, Germany
+ * Copyright (C) 1997 - 2000  Heinz Mauelshagen, Sistina Software
  *
  * March,October,November 1997
- * January 1999
+ * January,August 1999
+ * February 2000
  *
  *
  * This LVM library is free software; you can redistribute it and/or
@@ -30,31 +31,32 @@
  *    08/10/1997 - changed partition number to absolute
  *    07/11/1997 - added debug info
  *    23/01/1999 - extension for devfs names
+ *    15/08/1999 - rewritten based on lvm_dir_cache_find()
+ *    31/01/2000 - use debug_enter()/debug_leave()
  *
  */
 
 #include <liblvm.h>
 
 int pv_check_part ( char *dev_name) {
-   int i = 0;
    int ret = 0;
-   char *part_ptr = NULL;
+   dir_cache_t *cache_entry = NULL;
 
 #ifdef DEBUG
-   debug ( "pv_check_part -- CALLED with %s\n", dev_name);
+   debug_enter ( "pv_check_part -- CALLED with %s\n", dev_name);
 #endif
 
    if ( dev_name == NULL || pv_check_name ( dev_name) < 0) ret = -LVM_EPARAM;
    else {
-      part_ptr = dev_name + ( i = strlen ( dev_name)) - 1;
-      for ( ; i > 0; i--, part_ptr--) if ( !isdigit ( *part_ptr)) break;
-      part_ptr++;
-      ret = atoi ( part_ptr);
+      if ( ( cache_entry = lvm_dir_cache_find ( dev_name)) != NULL) {
+         ret = MINOR ( cache_entry->st_rdev) %
+               lvm_partition_count ( cache_entry->st_rdev);
+      } else ret = -1;
       if ( ret < MIN_PART || ret > MAX_PART) ret = -LVM_EPV_CHECK_PART;
    }
 
 #ifdef DEBUG
-   debug ( "pv_check_part -- LEAVING with %d\n", ret);
+   debug_leave ( "pv_check_part -- LEAVING with %d\n", ret);
 #endif
    return ret;
 }
