@@ -1,7 +1,7 @@
 /*
  * tools/lib/pv_move.c
  *
- * Copyright (C) 1997, 1998, 1999  Heinz Mauelshagen, Germany
+ * Copyright (C) 1997, 1998, 1999  Heinz Mauelshagen, Sistina Software
  *
  * May  1998
  * July 1998
@@ -437,7 +437,7 @@ pv_move_pe_continue:
 
    if ( opt_v > 1) {
       printf ( "lv: %s[%d]  old_dev: %02d:%02d  new_dev: %02d:%02d  "
-               "old_pe_sector: %u  new_pe_sector: %u\n",
+               "old_pe_sector: %lu  new_pe_sector: %lu\n",
                le_remap_req.lv_name,
                vg->pv[dst_pv_index]->pe[pe_dest].lv_num,
                MAJOR ( le_remap_req.old_dev),
