@@ -1,11 +1,12 @@
 /*
  * tools/lib/vg_read_with_pv_and_lv.c
  *
- * Copyright (C) 1997 - 1999  Heinz Mauelshagen, Germany
+ * Copyright (C) 1997 - 2000  Heinz Mauelshagen, Sistina Software
  *
  * March,October,November 1997
  * June,August 1998
  * January,March 1999
+ * January 2000
  *
  *
  * This LVM library is free software; you can redistribute it and/or
@@ -32,6 +33,7 @@
  *                 in vg_read_with_pv_and_lv() from pv_read_all_pv cache
  *    20/03/1999 - added major device number correction in case
  *                 of old block device major 
+ *    08/02/2000 - use debug_enter()/debug_leave()
  *
  */
 
@@ -47,47 +49,55 @@ int vg_read_with_pv_and_lv ( char *vg_name, vg_t **vg) {
    int ret = 0;
    int size = 0;
    uint pe_index = 0;
-   disk_pe_t **pe = NULL;
+   pe_disk_t **pe = NULL;
    lv_t **lv = NULL;
    vg_t *vg_this = NULL;
    pv_t **pv = NULL;
 
 #ifdef DEBUG
-   debug ( "vg_read_with_pv_and_lv -- CALLED\n");
+   debug_enter ( "vg_read_with_pv_and_lv -- CALLED\n");
 #endif
 
    if ( vg_name == NULL || vg == NULL ||
-        vg_check_name ( vg_name) < 0) return -LVM_EPARAM;
+        vg_check_name ( vg_name) < 0) {
+      ret = -LVM_EPARAM;
+      goto vg_read_with_pv_and_lv_end;
+   }
 
    *vg = NULL;
    if ( ( ret = vg_read ( vg_name, &vg_this)) < 0 &&
-        ret != -LVM_EVG_READ_VG_EXPORTED) return ret;
+        ret != -LVM_EVG_READ_VG_EXPORTED)
+      goto vg_read_with_pv_and_lv_end;
 
 #ifdef DEBUG
    debug ( "vg_read_with_pv_and_lv -- AFTER vg_read; "
             " vg_this->lv_cur: %lu\n", vg_this->lv_cur);
-   debug ( "vg_read_with_pv_and_lv -- vg_show AFTER first read\n");
-   if ( opt_d > 0) vg_show ( vg_this);
 #endif
 
-   if ( vg_this->pv_cur == 0) return -LVM_EVG_READ_WITH_PV_AND_LV_PV_CUR;
+   if ( vg_this->pv_cur == 0) {
+      ret = -LVM_EVG_READ_WITH_PV_AND_LV_PV_CUR;
+      goto vg_read_with_pv_and_lv_end;
+   }
 
    if ( ( ret = pv_read_all_pv_of_vg ( vg_name, &pv, FALSE)) < 0)
-      return ret;
+      goto vg_read_with_pv_and_lv_end;
+
 #ifdef DEBUG
    debug ( "vg_read_with_pv_and_lv -- AFTER pv_read_all_pv_of_vg\n");
 #endif
 
    if ( ( ret = pv_read_all_pe_of_vg ( vg_name, &pe, FALSE)) < 0)
-      return ret;
+      goto vg_read_with_pv_and_lv_end;
+
 #ifdef DEBUG
    debug ( "vg_read_with_pv_and_lv -- AFTER pv_read_all_pe_of_vg\n");
 #endif
 
-   if ( ( ret = lv_read_all_lv_of_vg ( vg_name, &lv, FALSE)) < 0)
-      return ret;
+   if ( ( ret = lv_read_all_lv ( vg_name, &lv, FALSE)) < 0)
+      goto vg_read_with_pv_and_lv_end;
+
 #ifdef DEBUG
-   debug ( "vg_read_with_pv_and_lv -- AFTER lv_read_all_lv_of_vg;"
+   debug ( "vg_read_with_pv_and_lv -- AFTER lv_read_all_lv;"
             " vg_this->pv_cur: %lu  vg_this->pv_max: %lu  ret: %d\n",
             vg_this->pv_cur, vg_this->pv_max, ret);
 #endif
@@ -102,15 +112,17 @@ int vg_read_with_pv_and_lv ( char *vg_name, vg_t **vg) {
             fprintf ( stderr, "malloc error in %s [line %d]\n",
                               __FILE__, __LINE__);
             vg_free ( vg_this, FALSE);
-            return -LVM_EVG_READ_WITH_PV_AND_LV_MALLOC;
+            ret = -LVM_EVG_READ_WITH_PV_AND_LV_MALLOC;
+            goto vg_read_with_pv_and_lv_end;
          }
          memcpy ( vg_this->pv[p], pv[p], sizeof ( pv_t));
-         size = vg_this->pv[p]->pe_total * sizeof ( disk_pe_t);
+         size = vg_this->pv[p]->pe_total * sizeof ( pe_disk_t);
          if ( ( vg_this->pv[p]->pe = malloc ( size)) == NULL) {
             fprintf ( stderr, "malloc error in %s [line %d]\n",
                               __FILE__, __LINE__);
             vg_free ( vg_this, FALSE);
-            return -LVM_EVG_READ_WITH_PV_AND_LV_MALLOC;
+            ret = -LVM_EVG_READ_WITH_PV_AND_LV_MALLOC;
+            goto vg_read_with_pv_and_lv_end;
          }
          memcpy ( vg_this->pv[p]->pe, pe[p], size);
       }
@@ -131,7 +143,8 @@ int vg_read_with_pv_and_lv ( char *vg_name, vg_t **vg) {
                   fprintf ( stderr, "malloc error in %s [line %d]\n",
                                     __FILE__, __LINE__);
                   vg_free ( vg_this, FALSE);
-                  return -LVM_EVG_READ_WITH_PV_AND_LV_MALLOC;
+                  ret = -LVM_EVG_READ_WITH_PV_AND_LV_MALLOC;
+                  goto vg_read_with_pv_and_lv_end;
                }
                memcpy ( vg_this->lv[l], lv[l], sizeof ( lv_t));
 #ifdef DEBUG
@@ -145,7 +158,8 @@ int vg_read_with_pv_and_lv ( char *vg_name, vg_t **vg) {
                   fprintf ( stderr, "malloc error in %s [line %d]\n",
                                     __FILE__, __LINE__);
                   vg_free ( vg_this, FALSE);
-                  return -LVM_EVG_READ_WITH_PV_AND_LV_MALLOC;
+                  ret = -LVM_EVG_READ_WITH_PV_AND_LV_MALLOC;
+                  goto vg_read_with_pv_and_lv_end;
                }
                /* construct the lv_current_pe pointer array */
                p = npe = 0;
@@ -158,15 +172,19 @@ int vg_read_with_pv_and_lv ( char *vg_name, vg_t **vg) {
                            vg_this->pv[p]->pv_dev;
                         vg_this->lv[l]->lv_current_pe[pe_index].pe =
 			   LVM_PE_DISK_OFFSET( ope, vg_this->pv[p]);
+                        vg_this->lv[l]->lv_current_pe[pe_index].reads = \
+                        vg_this->lv[l]->lv_current_pe[pe_index].writes = 0;
                         npe++;
                      }
                   }
                }
-               if ( npe != vg_this->lv[l]->lv_allocated_le)
-                  return -LVM_EVG_READ_WITH_PV_AND_LV_LV_ALLOCATED_LE;
-               /* correct LVM_BLOCK_MAJOR */
-               if ( MAJOR ( vg_this->lv[l]->lv_dev) != LVM_BLOCK_MAJOR) {
-                  vg_this->lv[l]->lv_dev = ( LVM_BLOCK_MAJOR << MINORBITS) +
+               if ( npe != vg_this->lv[l]->lv_allocated_le) {
+                  ret = -LVM_EVG_READ_WITH_PV_AND_LV_LV_ALLOCATED_LE;
+                  goto vg_read_with_pv_and_lv_end;
+               }
+               /* correct LVM_BLK_MAJOR */
+               if ( MAJOR ( vg_this->lv[l]->lv_dev) != LVM_BLK_MAJOR) {
+                  vg_this->lv[l]->lv_dev = ( LVM_BLK_MAJOR << MINORBITS) +
                                            MINOR ( vg_this->lv[l]->lv_dev);
                }
             } else vg_this->lv[l] = NULL;
@@ -176,8 +194,9 @@ int vg_read_with_pv_and_lv ( char *vg_name, vg_t **vg) {
 
    if ( ret == 0) *vg = vg_this;
 
+vg_read_with_pv_and_lv_end:
 #ifdef DEBUG
-   debug ( "vg_read_with_pv_and_lv -- LEAVING %d\n", ret);
+   debug_leave ( "vg_read_with_pv_and_lv -- LEAVING with ret: %d\n", ret);
 #endif
    return ret;
 }
