@@ -1,12 +1,12 @@
 /*
  * tools/lvchange.c
  *
- * Copyright (C) 1997 - 2000  Heinz Mauelshagen, Germany
+ * Copyright (C) 1997 - 2000  Heinz Mauelshagen, Sistina Software
  *
  * May,October 1997
  * April-June,August 1998
  * February,July,August,October 1999
- * February 2000
+ * February,November 2000
  *
  * LVM is free software; you can redistribute it and/or modify
  * it under the terms of the GNU General Public License as published by
@@ -41,6 +41,7 @@
  *                 snapshot logical volumes
  *    06/10/1999 - implemented support for long options
  *    15/02/2000 - use lvm_error()
+ *    07/11/2000 - used LVM_VG_NAME environment variable
  *
  */
 
@@ -171,7 +172,7 @@ int main ( int argc, char **argv)
                      "\t[-p/--permission r/rw]\n"
                      "\t[-r/--readahead ReadAheadSectors]\n"
                      "\t[-v/--verbose]\n"
-                     "\tLogicalVolumePath [LogicalVolumePath...]\n\n",
+                     "\tLogicalVolume[Path] [LogicalVolume[Path]...]\n\n",
                      lvm_version, LVM_LIB_IOP_VERSION,  cmd, cmd);
             return 0;
             break;
@@ -241,8 +242,12 @@ int main ( int argc, char **argv)
 
    /* walk through logical volumes */
    for ( ; optind < argc; optind++) {
+      char buffer[NAME_LEN];
+
       lv_name = argv[optind];
 
+      LVM_CHECK_DEFAULT_VG_NAME ( lv_name, buffer, NAME_LEN);
+
       if ( opt_v > 0) printf ( "%s -- checking logical volume name\n", cmd);
       if ( lv_check_name ( lv_name) < 0) {
          fprintf ( stderr, "%s -- invalid logical volume name \"%s\"\n\n",
@@ -250,17 +255,18 @@ int main ( int argc, char **argv)
          continue;
       }
    
-      vg_name = vg_name_of_lv ( lv_name);
-   
       /* does VG exist? */
+      vg_name = vg_name_of_lv ( lv_name);
       if ( opt_v > 0) printf ( "%s -- checking volume group existence\n", cmd);
       if ( lvm_tab_vg_check_exist ( vg_name, NULL) != TRUE) {
-         if(vg_name)
+         if ( vg_name != NULL) {
             fprintf ( stderr, "%s -- can't change logical volume: volume group "
-                      "\"%s\" doesn't exist\n\n", cmd, vg_name);
-         else
-            fprintf ( stderr, "%s -- can't change logical volume: need to "
-                      "specify volume group\n\n", cmd);
+                 "\"%s\" doesn't exist\n\n", cmd, vg_name);
+         } else {
+            fprintf ( stderr, "%s -- can't change logical volume: "
+                              "need to specify volume group\n\n",
+                              cmd);
+         }
          return LVM_ELVCHANGE_VG_CHECK_EXIST;
       }
 
@@ -286,15 +292,6 @@ int main ( int argc, char **argv)
          continue;
       }
    
-      if ( opt_v > 0) printf ( "%s -- checking logical volume \"%s\" "
-                               "existence\n",
-                               cmd, lv_name);
-      if ( ( ret = lvm_tab_lv_check_exist ( lv_name)) != TRUE) {
-         fprintf ( stderr, "%s -- logical volume \"%s\" doesn't exist\n\n",
-                   cmd, lv_name);
-         continue;
-      }
-   
       if ( opt_v > 0) printf ( "%s -- checking consistency of \"%s\"\n",
                                cmd, vg_name);
       if ( ( ret = vg_check_consistency_with_pv_and_lv ( vg)) < 0) {
@@ -304,6 +301,15 @@ int main ( int argc, char **argv)
          continue;
       }
       
+      if ( opt_v > 0) printf ( "%s -- checking logical volume \"%s\" "
+                               "existence\n",
+                               cmd, lv_name);
+      if ( ( ret = lvm_tab_lv_check_exist ( lv_name)) != TRUE) {
+         fprintf ( stderr, "%s -- logical volume \"%s\" doesn't exist\n\n",
+                   cmd, lv_name);
+         continue;
+      }
+
       if ( opt_v > 0) printf ( "%s -- getting index of logical volume \"%s\" "
                                "in volume group \"%s\"\n",
                                cmd, lv_name, vg_name);
