@@ -1,11 +1,12 @@
 /*
  * tools/lvextend.c
  *
- * Copyright (C) 1997 - 1999  Heinz Mauelshagen, Germany
+ * Copyright (C) 1997 - 2000  Heinz Mauelshagen, Sistina Software
  *
  * April-May 1997
  * May,June,September 1998
- * February 1999
+ * February,August,October 1999
+ * February,July,November 2000
  *
  * LVM is free software; you can redistribute it and/or modify
  * it under the terms of the GNU General Public License as published by
@@ -36,6 +37,11 @@
  *    04/09/1998 - corrected some messages
  *    06/02/1999 - fixed lvm_check_number() usage
  *    05/07/1999 - avoided vg_status()
+ *    15/08/1999 - enhanced to avoid extension of snapshot logical volumes
+ *    06/10/1999 - implemented support for long options
+ *    15/02/2000 - use lvm_error()
+ *    01/07/2000 - support for snapshot logical volumes
+ *    07/11/2000 - used LVM_VG_NAME environment variable
  *
  */
 
@@ -60,22 +66,36 @@ int main ( int argc, char **argv) {
    int l = 0;
    int pa = 0;
    int opt_A = 1;
+   int opt_A_set = 0;
    int opt_l = 0;
    int opt_L = 0;
    int opt_v = 0;
    int ret = 0;
    int sign = 0;
    int size_rest = 0;
-   uint new_size = 0;
+   long new_size = 0;
+   char buffer[NAME_LEN];
    char *dummy = 0;
 #ifdef DEBUG
    char *options = "A:dh?l:L:v";
 #else
    char *options = "A:h?l:L:v";
 #endif
+   struct option long_options[] = {
+      { "autobackup", required_argument, NULL, 'A'},
+#ifdef DEBUG
+      { "debug",      no_argument,       NULL, 'd'},
+#endif
+      { "help",       no_argument,       NULL, 'h'},
+      { "extents",    required_argument, NULL, 'l'},
+      { "size",       required_argument, NULL, 'L'},
+      { "verbose",    no_argument,       NULL, 'v'},
+      { NULL, 0, NULL, 0}
+   };
    char *lv_name = NULL;
    char *vg_name = NULL;
    char **pv_allowed = NULL;
+   lv_t *lv = NULL;
    vg_t *vg = NULL;
 
    cmd = basename ( argv[0]);
@@ -83,9 +103,11 @@ int main ( int argc, char **argv) {
    SUSER_CHECK;
    LVMTAB_CHECK;
 
-   while ( ( c = getopt ( argc, argv, options)) != EOF) {
+   while ( ( c = getopt_long ( argc, argv, options,
+                               long_options, NULL)) != EOF) {
       switch ( c) {
          case 'A':
+	    opt_A_set++;
             if ( opt_A > 1) {
                fprintf ( stderr, "%s -- A option already given\n\n", cmd);
                return LVM_EINVALID_CMD_LINE;
@@ -111,20 +133,20 @@ int main ( int argc, char **argv) {
 
          case 'h':
          case '?':
-            printf ( "\n%s\n\n%s -- Logical Volume Extend\n\n"
+            printf ( "\n%s (IOP %d)\n\n%s -- Logical Volume Extend\n\n"
                      "Synopsis:\n"
                      "---------\n\n"
                      "%s\n"
-                     "\t[-A y/n]\n"
+                     "\t[-A/--autobackup y/n]\n"
 #ifdef DEBUG
-                     "\t[-d]\n"
+                     "\t[-d/--debug]\n"
 #endif
-                     "\t[-h/?]\n"
-                     "\t{-l [+]LogicalExtentsNumber |\n"
-                     "\t -L [+]LogicalVolumeSize[kKmMgGtT]}\n"
-                     "\t[-v]\n"
-                     "\tLogicalVolumePath [ PhysicalVolumePath... ]\n\n",
-                     lvm_version, cmd, cmd);
+                     "\t[-h/-?/--help]\n"
+                     "\t{-l/--extents [+]LogicalExtentsNumber |\n"
+                     "\t -L/--size [+]LogicalVolumeSize[kKmMgGtT]}\n"
+                     "\t[-v/--verbose]\n"
+                     "\tLogicalVolume[Path] [ PhysicalVolumePath... ]\n\n",
+                     lvm_version, LVM_LIB_IOP_VERSION,  cmd, cmd);
             return 0;
             break;
 
@@ -180,12 +202,14 @@ int main ( int argc, char **argv) {
    }
 
    CMD_MINUS_CHK
+   CMD_CHECK_OPT_A_SET;
 
    if ( optind == argc) {
       fprintf ( stderr, "%s -- please enter a logical volume path\n\n", cmd);
       return LVM_ELVEXTEND_LV_MISSING;
    }
    lv_name = argv[optind];
+   LVM_CHECK_DEFAULT_VG_NAME ( lv_name, buffer, sizeof ( buffer));
    optind++;
 
    /* physical volumes in command line */
@@ -265,6 +289,16 @@ int main ( int argc, char **argv) {
       return LVM_ELVEXTEND_LV_GET_INDEX;
    }
 
+
+   if ( opt_v > 0) printf ( "%s -- getting status of \"%s\" from VGDA in "
+                            "kernel\n", cmd, lv_name);
+   if ( ( ret = lv_status_byname ( vg_name, lv_name, &lv)) < 0) {
+      fprintf ( stderr, "%s -- ERROR \"%s\" getting status of logical "
+                        "volume \"%s\"\n\n",
+                        cmd, lvm_error ( ret), lv_name);
+      return LVM_ELVEXTEND_LV_STATUS_BYNAME;
+   }
+
    if ( opt_L + opt_l > 0) {
       if ( opt_L > 0) new_size *= 2;
       else            new_size *= vg->pe_size;
@@ -277,13 +311,14 @@ int main ( int argc, char **argv) {
 
       size_rest = new_size % ( vg->lv[l]->lv_stripes * vg->pe_size);
       if ( size_rest != 0) {
-         printf ( "%s -- rounding size %d KB to stripe boundary size ",
+         printf ( "%s -- rounding size %ld KB to stripe boundary size ",
                   cmd, new_size / 2);
          new_size = new_size - size_rest +
                     vg->lv[l]->lv_stripes * vg->pe_size;
-         printf ( "%d KB\n", new_size / 2);
+         printf ( "%ld KB\n", new_size / 2);
       }
 
+
       if ( new_size <= vg->lv[l]->lv_size) {
          fprintf ( stderr, "%s -- new size %s is not bigger than old size\n\n",
                    cmd, ( dummy = lvm_show_size ( new_size / 2, SHORT)));
@@ -312,11 +347,21 @@ int main ( int argc, char **argv) {
    }
 
    if ( new_size > LVM_LV_SIZE_MAX( vg)) {
-      fprintf ( stderr, "%s -- new size %d is to large for VGDA in kernel\n\n",
+      fprintf ( stderr, "%s -- new size %ld is to large for VGDA in kernel\n\n",
                 cmd, new_size / 2);
       return LVM_ELVEXTEND_LV_SIZE_MAX;
    }
 
+   if ( lv->lv_access & LV_SNAPSHOT) {
+      ulong lv_org_size = vg->lv[lv_get_index_by_minor ( vg, vg->lv[l]->lv_snapshot_minor)]->lv_size;
+
+      if ( new_size > lv_org_size * 1.1) {
+            fprintf ( stderr, "%s -- ERROR: size of snapshot is too large\n",
+                              cmd);
+            return LVM_EINVALID_CMD_LINE;
+      }
+   }
+
    printf ( "%s -- extending logical volume \"%s\" to %s\n",
             cmd, lv_name, ( dummy = lvm_show_size ( new_size / 2, SHORT)));
    free ( dummy); dummy = NULL;
@@ -337,32 +382,45 @@ int main ( int argc, char **argv) {
                               cmd, lv_name);
          }
       } else {
-         fprintf ( stderr, "%s -- ERROR %d setting logical volume \"%s\" "
+         fprintf ( stderr, "%s -- ERROR \"%s\" setting logical volume \"%s\" "
                            " up for extend\n",
-                           cmd, ret, lv_name);
+                           cmd, lvm_error ( ret), lv_name);
       }
       fprintf ( stderr, "\n");
       return LVM_ELVEXTEND_LV_SETUP;
    }
 
+   if ( lv->lv_access & LV_SNAPSHOT) {
+      if ( ( ret = lv_setup_COW_table_for_create (
+                      vg,
+                      vg->lv[lv_get_index_by_minor ( vg, vg->lv[l]->lv_snapshot_minor)]->lv_name,
+                      l,
+                      lv->lv_chunk_size * SECTOR_SIZE / 1024)) < 0) {
+         fprintf ( stderr, "%s -- ERROR \"%s\" setting up snapshot "
+                           "copy on write exception table for \"%s\"\n\n",
+                           cmd, lvm_error ( ret), lv_name);
+         return LVM_ELVEXTEND_LV_SETUP_COW_TABLE_FOR_CREATE;
+      }
+   }
+
    lvm_dont_interrupt ( 0);
 
    /* create it in kernel */
    if ( opt_v > 0) printf ( "%s -- extending logical volume \"%s\" in "
                             "VGDA of kernel\n", cmd, lv_name);
    if ( ( ret = lv_extend ( vg, vg->lv[l], lv_name)) != 0) {
-      fprintf ( stderr, "%s -- ERROR %d extending logical volume \"%s\" "
+      fprintf ( stderr, "%s -- ERROR \"%s\" extending logical volume \"%s\" "
                         " in kernel's VGDA\n\n", 
-                cmd, ret, lv_name);
+                        cmd, lvm_error ( ret), lv_name);
       return LVM_ELVEXTEND_LV_EXTEND;
    }
 
    if ( opt_v > 0) printf ( "%s -- storing data of volume group \"%s\" "
                             "on disk(s)\n", cmd, vg_name);
    if ( ( ret = vg_write_with_pv_and_lv ( vg)) != 0) {
-      fprintf ( stderr, "%s -- ERROR %d storing data of volume group \"%s\" "
-                        "on disk(s)\n\n", 
-                cmd, ret, vg_name);
+      fprintf ( stderr, "%s -- ERROR \"%s\" storing data of volume group "
+                        "\"%s\" on disk(s)\n\n", 
+                        cmd, lvm_error ( ret), vg_name);
       return LVM_ELVEXTEND_VG_WRITE;
    }
 
@@ -372,6 +430,9 @@ int main ( int argc, char **argv) {
       printf ( "%s -- doing automatic backup of volume group \"%s\"\n",
                cmd, vg_name);
       vg_cfgbackup ( vg_name, VG_BACKUP_DIR, opt_v, vg);
+   } else {
+      printf ( "%s -- WARNING: you don't have an automatic backup of \"%s\"\n",
+               cmd, vg_name);
    }
 
    lvm_interrupt ();
