@@ -1,7 +1,7 @@
 /*
  * tools/lvmsadc.c
  *
- * Copyright (C) 1998 - 1999  Heinz Mauelshagen, Germany
+ * Copyright (C) 1998 - 1999  Heinz Mauelshagen, Sistina Software
  *
  * July,September 1998
  * February,October,November 1999
@@ -161,7 +161,7 @@ int main ( int argc, char **argv) {
                               vg->lv[l]->lv_name,
                               vg->lv[l]->lv_allocated_le);
                for ( le = 0; le < vg->lv[l]->lv_allocated_le; le++) {
-                  fprintf ( log, "%lu %u %u\n",
+                  fprintf ( log, "%lu %lu %lu\n",
                                  le,
                                  vg->lv[l]->lv_current_pe[le].reads,
                                  vg->lv[l]->lv_current_pe[le].writes);
