@@ -1,10 +1,11 @@
 /*
  * tools/lvscan.c
  *
- * Copyright (C) 1997 - 1999  Heinz Mauelshagen, Germany
+ * Copyright (C) 1997 - 2000  Heinz Mauelshagen, Sistina Software
  *
  * May,June,September 1998
- * January,February,May 1999
+ * January,February,May,October 1999
+ * February 2000
  *
  * LVM is free software; you can redistribute it and/or modify
  * it under the terms of the GNU General Public License as published by
@@ -32,6 +33,8 @@
  *    20/02/1999 - added LV total message
  *    21/02/1999 - remove LVM_LOCK and LVM_UNLOCK
  *    10/05/1999 - added total capacity display
+ *    06/10/1999 - implemented support for long options
+ *    15/02/2000 - use lvm_error()
  *
  */
 
@@ -59,10 +62,20 @@ int main ( int argc, char **argv)
    int vg_total = 0;
    char *dummy = NULL;
 #ifdef DEBUG
-   char *options = "AbdDh?v";
+   char *options = "bdDh?v";
 #else
-   char *options = "AbDh?v";
+   char *options = "bDh?v";
 #endif
+   struct option long_options[] = {
+      { "blockdevice", no_argument, NULL, 'b'},
+#ifdef DEBUG
+      { "debug",       no_argument, NULL, 'd'},
+#endif
+      { "disk",        no_argument, NULL, 'D'},
+      { "help",        no_argument, NULL, 'h'},
+      { "verbose",     no_argument, NULL, 'v'},
+      { NULL, 0, NULL, 0}
+   };
    char *vg_name = NULL;
    vg_t *vg = NULL;
 
@@ -71,7 +84,8 @@ int main ( int argc, char **argv)
    SUSER_CHECK;
    LVMTAB_CHECK;
 
-   while ( ( c = getopt ( argc, argv, options)) != EOF) {
+   while ( ( c = getopt_long ( argc, argv, options,
+                               long_options, NULL)) != EOF) {
       switch ( c) {
 
 #ifdef DEBUG
@@ -102,18 +116,18 @@ int main ( int argc, char **argv)
 
          case 'h':
          case '?':
-            printf ( "\n%s\n\n%s -- Logical Volume Scan\n\n"
+            printf ( "\n%s (IOP %d)\n\n%s -- Logical Volume Scan\n\n"
                      "Synopsis:\n"
                      "---------\n\n"
                      "%s\n"
-                     "\t[-b]\n"
+                     "\t[-b/--blockdevice]\n"
 #ifdef DEBUG
-                     "\t[-d]\n"
+                     "\t[-d/--debug]\n"
 #endif
-                     "\t[-D]\n"
-                     "\t[-h/?]\n"
-                     "\t[-v]\n\n",
-                     lvm_version, cmd, cmd);
+                     "\t[-D/--disk]\n"
+                     "\t[-h/-?/--help]\n"
+                     "\t[-v/--verbose]\n\n",
+                     lvm_version, LVM_LIB_IOP_VERSION,  cmd, cmd);
             return 0;
             break;
 
@@ -181,12 +195,11 @@ int main ( int argc, char **argv)
                if ( opt_v > 0) printf ( "%s -- getting VGDA of volume "
                                         "group \"%s\" from kernel\n",
                                         cmd, vg_name);
-               if ( ( ret = vg_status_with_pv_and_lv ( vg_name, &vg,
-                                                       TRUE)) < 0) {
+               if ( ( ret = vg_status_with_pv_and_lv ( vg_name, &vg)) < 0) {
                   fprintf ( stderr,
-                            "%s -- ERROR %d getting status of volume "
+                            "%s -- ERROR \"%s\" getting status of volume "
                             " group \"%s\"\n",
-                             cmd, ret, vg_name);
+                             cmd, lvm_error ( ret), vg_name);
                   return LVM_ELVSCAN_VG_STATUS;
                }
             } else {
@@ -207,9 +220,9 @@ int main ( int argc, char **argv)
                              cmd, vg_name);
                } else {
                   fprintf ( stderr,
-                            "%s -- ERROR %d getting data of volume "
+                            "%s -- ERROR \"%s\" getting data of volume "
                             "group \"%s\" form kernel\n",
-                            cmd, ret, vg_name);
+                            cmd, lvm_error ( ret), vg_name);
                }
                continue;
             }
@@ -219,12 +232,18 @@ int main ( int argc, char **argv)
             if ( vg->lv[l] != NULL &&
                  lv_check_name ( vg->lv[l]->lv_name) == 0) {
                printf ( "%s -- ", cmd);
-               if ( lv_check_active ( vg_name, vg->lv[l]->lv_name) != TRUE)
+               if ( lv_check_active ( vg_name, vg->lv[l]->lv_name) != TRUE || opt_D != 0)
                   printf ( "inactive");
                else {
                   printf ( "ACTIVE  ");
                   lv_active++;
                }
+               if ( vg->lv[l]->lv_access & LV_SNAPSHOT_ORG)
+                  printf ( "Original");
+               else if ( vg->lv[l]->lv_access & LV_SNAPSHOT)
+                  printf ( "Snapshot");
+               else
+                  printf ( "        ");
                printf ( " \"%s\" [%s]",
                         vg->lv[l]->lv_name,
                         ( dummy = lvm_show_size ( vg->lv[l]->lv_size / 2,
