@@ -1,11 +1,12 @@
 /*
  * tools/pvcreate.c
  *
- * Copyright (C) 1997 - 1999  Heinz Mauelshagen, Germany
+ * Copyright (C) 1997 - 2000  Heinz Mauelshagen, Sistina Software
  *
  * March, October-November 1997
  * May,June,September 1998
- * January,June 1999
+ * January,June,September,October 1999
+ * February 2000
  *
  * LVM is free software; you can redistribute it and/or modify
  * it under the terms of the GNU General Public License as published by
@@ -37,6 +38,12 @@
  *    13/01/1999 - avoided error message for uninitialized physical volume
  *    22/06/1999 - added recongnition of Win98 extended partitions
  *                 by Steve Brueggeman
+ *    22/09/1999 - added support for new partition identifier 0xDE
+ *    24/09/1999 - stop in case of a physical volume error instead of
+ *                 continuing with the next one in line
+ *               - added new exit codes
+ *    06/10/1999 - implemented support for long options
+ *    15/02/2000 - use lvm_error()
  *
  */
 
@@ -64,6 +71,16 @@ int main ( int argc, char **argv)
 #else
    char *options = "fh?yv";
 #endif
+   struct option long_options[] = {
+#ifdef DEBUG
+      { "debug",   no_argument, NULL, 'd'},
+#endif
+      { "force",   no_argument, NULL, 'f'},
+      { "help",    no_argument, NULL, 'h'},
+      { "yes",     no_argument, NULL, 'y'},
+      { "verbose", no_argument, NULL, 'v'},
+      { NULL, 0, NULL, 0}
+   };
    char *pv_name = NULL;
    pv_t *pv = NULL;
    pv_t pv_new;
@@ -74,7 +91,8 @@ int main ( int argc, char **argv)
    SUSER_CHECK;
    LVMTAB_CHECK;
 
-   while ( ( c = getopt ( argc, argv, options)) != EOF) {
+   while ( ( c = getopt_long ( argc, argv, options,
+                               long_options, NULL)) != EOF) {
       switch ( c) {
 #ifdef DEBUG
          case 'd':
@@ -96,19 +114,19 @@ int main ( int argc, char **argv)
 
          case 'h':
          case '?':
-            printf ( "\n%s\n\n%s -- Physical Volume Create\n\n"
+            printf ( "\n%s (IOP %d)\n\n%s -- Physical Volume Create\n\n"
                      "Synopsis:\n"
                      "---------\n\n"
                      "%s\n"
 #ifdef DEBUG
-                     "\t[-d]\n"
+                     "\t[-d/--debug]\n"
 #endif
-                     "\t[-f[f]]\n"
-                     "\t[-h/-?]\n"
-                     "\t[-y]\n"
-                     "\t[-v]\n"
+                     "\t[-f[f]/--force [--force]]\n"
+                     "\t [-h/-?/--help]\n"
+                     "\t[-y/--yes]\n"
+                     "\t[-v/--verbose]\n"
                      "\tPhysicalVolumePath [PhysicalVolumePath...]\n\n",
-                     lvm_version, cmd, cmd);
+                     lvm_version, LVM_LIB_IOP_VERSION,  cmd, cmd);
             return 0;
             break;
 
@@ -160,15 +178,15 @@ int main ( int argc, char **argv)
       if ( pv_check_name ( pv_name) < 0) {
          fprintf ( stderr, "%s -- invalid physical volume name \"%s\"\n",
                             cmd, pv_name);
+         return LVM_EPVCREATE_PV_CHECK_NAME;
          continue;
       }
 
       if ( opt_v > 0) printf ( "%s -- getting physical volume size\n", cmd);
       if ( ( size = pv_get_size ( pv_name, &part)) < 0) {
          if ( size == -LVM_EPV_GET_SIZE_PART) {
-            fprintf ( stderr, "%s -- partition \"%s\" doesn't exist\n",
+            fprintf ( stderr, "%s -- device \"%s\" has a partition table\n",
                               cmd, pv_name);
-            continue;
          } else if ( size == -LVM_EPV_GET_SIZE_OPEN) {
             fprintf ( stderr, "%s -- can't open physical volume \"%s\" to "
                               "get its size\n",
@@ -176,43 +194,54 @@ int main ( int argc, char **argv)
          } else if ( size == -LVM_EPV_GET_SIZE_NO_EXTENDED) {
             fprintf ( stderr, "%s -- extended partition \"%s\" doesn't exist\n",
                               cmd, pv_name);
+         } else if ( size == -LVM_EPV_GET_SIZE_LVM_DIR_CACHE) {
+            fprintf ( stderr, "%s -- device \"%s\" doesn't exist\n",
+                              cmd, pv_name);
          } else {
-            fprintf ( stderr, "%s -- ERROR %d getting size of physical "
+            fprintf ( stderr, "%s -- ERROR \"%s\" getting size of physical "
                               "volume \"%s\"\n",
-                              cmd, size, pv_name);
+                              cmd, lvm_error ( size), pv_name);
          }
-         continue;
+         fputc ( '\n', stderr);
+         return LVM_EPVCREATE_PV_GET_SIZE;
       }
 
       if ( opt_v > 0) printf ( "%s -- checking partition type\n", cmd);
       if ( part.sys_ind != 0) {
          if ( part.sys_ind == DOS_EXTENDED_PARTITION) {
-            fprintf ( stderr, "%s -- can't use DOS extended partition \"%s\"\n",
+            fprintf ( stderr, "%s -- can't use DOS extended partition "
+                              "\"%s\"\n\n",
                               cmd, pv_name);
-            continue;
+            return LVM_EPVCREATE_INVALID_ID;
 #ifdef LINUX_SWAP_PARTITION
          } else if ( part.sys_ind == LINUX_SWAP_PARTITION) {
-            fprintf ( stderr, "%s -- can't use Linux SWAP partition \"%s\"\n",
+            fprintf ( stderr, "%s -- can't use Linux SWAP partition \"%s\"\n\n",
                               cmd, pv_name);
-            continue;
+            return LVM_EPVCREATE_INVALID_ID;
 #endif
 #ifdef WIN98_EXTENDED_PARTITION
          } else if ( part.sys_ind == WIN98_EXTENDED_PARTITION) {
             fprintf ( stderr, "%s -- can't use Windows 98 extended partition "
-                              "\"%s\"\n",
+                              "\"%s\"\n\n",
                               cmd, pv_name);
-            continue;
+            return LVM_EPVCREATE_INVALID_ID;
 #endif
          } else if ( part.sys_ind == LINUX_EXTENDED_PARTITION) {
             fprintf ( stderr, "%s -- can't use Linux extended partition "
-                              "\"%s\"\n",
+                              "\"%s\"\n\n",
                               cmd, pv_name);
-            continue;
-         } else if ( part.sys_ind != LVM_PARTITION) {
+            return LVM_EPVCREATE_INVALID_ID;
+         } else if ( part.sys_ind != LVM_PARTITION &&
+                     part.sys_ind != LVM_NEW_PARTITION) {
             fprintf ( stderr, "%s -- invalid partition type 0x%x for \"%s\""
-                              " (must be 0x%x)\n", cmd, part.sys_ind,
-                              pv_name, LVM_PARTITION);
-            continue;
+                              " (must be 0x%x)\n\n",
+                              cmd, part.sys_ind, pv_name, LVM_NEW_PARTITION);
+            return LVM_EPVCREATE_INVALID_ID;
+         } else if ( part.sys_ind == LVM_PARTITION) {
+            fprintf ( stderr, "%s -- please change old LVM partition type on "
+                              "\"%s\" to 0x%x and retry\n\n",
+                              cmd, pv_name, LVM_NEW_PARTITION);
+            if ( opt_f < 2) return LVM_EPVCREATE_INVALID_ID;
          }
       }
    
@@ -275,21 +304,17 @@ int main ( int argc, char **argv)
       if ( opt_f > 0) {
          printf ( "%s -- WARNING: forcing physical volume creation on \"%s\"",
                   cmd, pv_name);
-         if ( pv_check_new == FALSE) printf ( " of volume group \"%s\"",
-                                              pv->vg_name);
+         if ( pv_check_new ( pv) == FALSE) printf ( " of volume group \"%s\"",
+                                                     pv->vg_name);
          printf ( "\n");
       }
    
-      if ( opt_v > 0) printf ( "%s -- checking for new physical volume\n", cmd);
-      if ( pv_check_new ( pv) == TRUE)
-         printf ( "%s -- reinitializing physical volume\n", cmd);
-   
       if ( lvm_tab_vg_check_exist ( pv->vg_name, NULL) == TRUE) {
          printf ( "%s -- removing lvmtab entry\n", cmd);
          if ( ( ret = lvm_tab_vg_remove ( pv->vg_name)) < 0) {
-            fprintf ( stderr, "%s -- ERROR %d removing volume group \"%s\" "
+            fprintf ( stderr, "%s -- ERROR \"%s\" removing volume group \"%s\" "
                               "from \"%s\"\n\n",
-                              cmd, ret, pv->vg_name, LVMTAB);
+                              cmd, lvm_error ( ret), pv->vg_name, LVMTAB);
             return LVM_EPVCREATE_VG_CHECK_EXIST;
          }
       }
@@ -300,9 +325,9 @@ int main ( int argc, char **argv)
                                "%s with %u sectors\n",
                                cmd, pv_name, size);
       if ( ( ret = pv_setup_for_create ( pv_name, &pv_new, size)) < 0) {
-         fprintf ( stderr, "%s -- ERROR %d setting up  physical "
-                           "volume \"%s\"\n",
-                           cmd, ret, pv_name);
+         fprintf ( stderr, "%s -- ERROR \"%s\" setting up  physical "
+                           "volume \"%s\"\n\n",
+                           cmd, lvm_error ( ret), pv_name);
          return LVM_EPVCREATE_PV_SETUP;
       }
    
@@ -315,8 +340,9 @@ int main ( int argc, char **argv)
          printf ( "%s -- physical volume \"%s\" successfully created\n",
                    cmd, pv_name);
       } else {
-         fprintf ( stderr, "%s -- ERROR %d creating physical volume \"%s\"\n",
-                   cmd, ret, pv_name);
+         fprintf ( stderr, "%s -- ERROR \"%s\" creating physical "
+                           "volume \"%s\"\n\n",
+                           cmd, lvm_error ( ret), pv_name);
          return LVM_EPVCREATE_PV_WRITE;
       }
       lvm_interrupt ();
