@@ -1,10 +1,12 @@
 /*
  * tools/vgcfgrestore.c
  *
- * Copyright (C) 1997 - 1999  Heinz Mauelshagen, Germany
+ * Copyright (C) 1997 - 2000  Heinz Mauelshagen, Sistina Software
  *
  * June,October 1997
  * May 1998
+ * October 1999
+ * February 2000
  *
  * LVM is free software; you can redistribute it and/or modify
  * it under the terms of the GNU General Public License as published by
@@ -17,7 +19,7 @@
  * GNU General Public License for more details.
  * 
  * You should have received a copy of the GNU General Public License
- * along with GNU CC; see the file COPYING.  If not, write to
+ * along with LVM; see the file COPYING.  If not, write to
  * the Free Software Foundation, 59 Temple Place - Suite 330,
  * Boston, MA 02111-1307, USA. 
  *
@@ -38,6 +40,9 @@
  *    02/05/1998 - used pv_check_new() instead of str-function
  *    16/05/1998 - added lvmtab checking
  *    21/05/1998 - implemented option -o
+ *    06/10/1999 - implemented support for long options
+ *    12/02/2000 - avoided catch 22 situation by avoiding lvmtab check 
+ *    15/02/2000 - use lvm_error()
  *
  */
 
@@ -67,6 +72,19 @@ int main ( int argc, char **argv)
 #else
    char *options = "f:h?ln:o:tv";
 #endif
+   struct option long_options[] = {
+#ifdef DEBUG
+      { "debug",      no_argument,       NULL, 'd'},
+#endif
+      { "file",       required_argument, NULL, 'f'},
+      { "help",       no_argument,       NULL, 'h'},
+      { "list",       no_argument,       NULL, 'l'},
+      { "name",       required_argument, NULL, 'n'},
+      { "oldpath",    required_argument, NULL, 'n'},
+      { "test",       no_argument,       NULL, 't'},
+      { "verbose",    no_argument,       NULL, 'v'},
+      { NULL, 0, NULL, 0}
+   };
    char *pv_name = NULL;
    char *pv_name_old = NULL;
    char *vg_name = NULL;
@@ -82,9 +100,9 @@ int main ( int argc, char **argv)
    memset ( &vg_write, 0, sizeof ( vg_write));
 
    SUSER_CHECK;
-   LVMTAB_CHECK;
 
-   while ( ( c = getopt ( argc, argv, options)) != EOF) {
+   while ( ( c = getopt_long ( argc, argv, options,
+                               long_options, NULL)) != EOF) {
       switch ( c) {
 #ifdef DEBUG
          case 'd':
@@ -107,22 +125,22 @@ int main ( int argc, char **argv)
 
          case 'h':
          case '?':
-            printf ( "\n%s\n\n%s -- Volume Group Configuration Restore\n\n"
+            printf ( "\n%s  (IOP %d)\n\n%s -- Volume Group Configuration Restore\n\n"
                      "Synopsis:\n"
                      "---------\n\n"
                      "%s\n"
 #ifdef DEBUG
-                     "\t[-d]\n"
+                     "\t[-d/--debug]\n"
 #endif
-                     "\t[-f VGConfPath]\n"
-                     "\t[-l[l]]\n"
-                     "\t[-n VolumeGroupName]\n"
-                     "\t[-h/-?]\n"
-                     "\t[-o OldPhysicalVolumePath]\n"
-                     "\t[-t]\n"
-                     "\t[-v]\n"
+                     "\t[-f/--file VGConfPath]\n"
+                     "\t[-l[l]/--list [--list]]\n"
+                     "\t[-n/--name VolumeGroupName]\n"
+                     "\t[-h/-?/--help]\n"
+                     "\t[-o/--oldpath OldPhysicalVolumePath]\n"
+                     "\t[-t/--test]\n"
+                     "\t[-v/--verbose]\n"
                      "\t[PhysicalVolumePath]\n\n",
-                     lvm_version, cmd, cmd);
+                     lvm_version, LVM_LIB_IOP_VERSION,  cmd, cmd);
             return 0;
             break;
 
@@ -236,8 +254,9 @@ int main ( int argc, char **argv)
                            "%s is not available\n\n",
                            cmd, vg_name);
       } else {
-         fprintf ( stderr, "%s -- ERROR %d restoring volume group \"%s\"\n\n",
-                           cmd, ret, vg_name);
+         fprintf ( stderr, "%s -- ERROR \"%s\" restoring volume group "
+                           "\"%s\"\n\n",
+                           cmd, lvm_error ( ret), vg_name);
       }
       return LVM_EVGCFGRESTORE_VG_CFGRESTORE;
    }
@@ -254,9 +273,9 @@ int main ( int argc, char **argv)
    if ( opt_v > 0) printf ( "%s -- checking volume group consistency"
                             " of \"%s\"\n", cmd, vg_name);
    if ( ( ret = vg_check_consistency_with_pv_and_lv ( &vg)) < 0) {
-      fprintf ( stderr, "%s -- ERROR %d: volume group \"%s\" "
+      fprintf ( stderr, "%s -- ERROR \"%s\" volume group \"%s\" "
                         "is inconsistent\n\n",
-                        cmd, ret, vg_name);
+                        cmd, lvm_error ( ret), vg_name);
       return LVM_EVGCFGRESTORE_VG_CHECK_CONSISTENCY;
    } else if ( opt_t > 0) printf ( "%s -- backup of volume group \"%s\" "
                                    " is consistent\n",
@@ -296,9 +315,9 @@ int main ( int argc, char **argv)
                                   cmd, pv_name);
          if ( ( ret = pv_read ( pv_name, &pv, NULL)) < 0 &&
               ret != -LVM_EPV_READ_MD_DEVICE) {
-            fprintf ( stderr, "%s -- ERROR %d reading physical "
+            fprintf ( stderr, "%s -- ERROR \"%s\" reading physical "
                               "volume \"%s\"\n\n",
-                              cmd, ret, pv_name);
+                              cmd, lvm_error ( ret), pv_name);
             return LVM_EVGCFGRESTORE_PV_READ;
          }
 
@@ -333,9 +352,9 @@ int main ( int argc, char **argv)
                               "volume \"%s\"\n",
                               cmd, vg_name, vg_write.pv[0]->pv_name);
          if ( ( ret = vg_write_with_pv_and_lv ( &vg_write)) < 0) {
-            fprintf ( stderr, "%s -- ERROR %d storing VGDA "
+            fprintf ( stderr, "%s -- ERROR \"%s\" storing VGDA "
                               "of \"%s\" on disks\n\n", 
-                              cmd, ret, vg_name);
+                              cmd, lvm_error ( ret), vg_name);
             return LVM_EVGCFGRESTORE_VG_WRITE;
          }
    
@@ -355,16 +374,16 @@ int main ( int argc, char **argv)
                                   cmd, vg_name, LVMTAB);
          if ( ( ret = lvm_tab_vg_insert ( vg_name)) < 0 &&
               ret != -LVM_ELVM_TAB_VG_INSERT_VG_EXISTS) {
-            fprintf ( stderr, "%s -- ERROR %d inserting volume group into "
+            fprintf ( stderr, "%s -- ERROR \"%s\" inserting volume group into "
                               "\"%s\"\n\n",
-                              cmd, ret, LVMTAB);
+                              cmd, lvm_error ( ret), LVMTAB);
             return LVM_EVGCFGRESTORE_LVM_TAB_VG_INSERT;
          }
          
          if ( ( ret = vg_cfgbackup ( vg_name, LVMTAB_DIR,
                                      opt_v, &vg)) < 0) {
-            fprintf ( stderr, "%s -- ERROR %d writing lvmtab\n\n",
-                              cmd, ret);
+            fprintf ( stderr, "%s -- ERROR \"%s\" writing lvmtab\n\n",
+                              cmd, lvm_error ( ret));
             return LVM_EVGCFGRESTORE_VG_CFGBACKUP;
          }
 
