@@ -1,7 +1,7 @@
 /*
  * tools/vgck.c
  *
- * Copyright (C) 1997 - 2000  Heinz Mauelshagen, Germany
+ * Copyright (C) 1997 - 2000  Heinz Mauelshagen, Sistina Software
  *
  * June 1998
  * October 1999
@@ -172,9 +172,6 @@ int main ( int argc, char **argv) {
             fprintf ( stderr, "%s -- ERROR \"%s\" checking existence of "
                               "volume group \"%s\"\n",
                               cmd, lvm_error ( ret), vg_name);
-         fprintf ( stderr, "%s -- trying to deactivate volume group "
-                           "\"%s\"\n", cmd, vg_name);
-         vg_deactivate ( vg_name);
       }
 
       if ( opt_v > 0) printf ( "%s -- reading volume group data for \"%s\" "
@@ -184,9 +181,6 @@ int main ( int argc, char **argv) {
                            "volume group \"%s\"\n",
                            cmd, lvm_error ( ret), vg_name);
          err_lvmtab = 1;
-         fprintf ( stderr, "%s -- trying to deactivate volume group "
-                           "\"%s\"\n", cmd, vg_name);
-         vg_deactivate ( vg_name);
       } else {
          if ( opt_v > 0) printf ( "%s -- checking volume group consistency "
                                   " of \"%s\" in lvmtab\n", cmd, vg_name);
@@ -194,9 +188,6 @@ int main ( int argc, char **argv) {
             fprintf ( stderr, "%s -- ERROR \"%s\" VGDA of \"%s\" is "
                               "INCONSISTENT in lvmtab\n",
                               cmd, lvm_error ( ret), vg_name);
-            fprintf ( stderr, "%s -- trying to deactivate volume group "
-                              "\"%s\"\n", cmd, vg_name);
-            vg_deactivate ( vg_name);
          } else {
             printf ( "%s -- VGDA of \"%s\" in lvmtab "
                      "is consistent\n", cmd, vg_name);
@@ -215,9 +206,6 @@ int main ( int argc, char **argv) {
             fprintf ( stderr, "%s -- please DON'T run vgscan prior to "
                               "vgcfgrestore\n", cmd);
          }
-         fprintf ( stderr, "%s -- trying to deactivate volume group "
-                           "\"%s\"\n", cmd, vg_name);
-         vg_deactivate ( vg_name);
       } else {
          if ( opt_v > 0) {
             printf ( "%s -- checking volume group consistency "
@@ -229,9 +217,6 @@ int main ( int argc, char **argv) {
                               "INCONSISTENT on physical volume%s\n",
                               cmd, lvm_error ( ret), vg_name,
                               vg->pv_cur > 1 ? "s" : "");
-            fprintf ( stderr, "%s -- trying to deactivate volume group "
-                              "\"%s\"\n", cmd, vg_name);
-            vg_deactivate ( vg_name);
          } else {
             printf ( "%s -- VGDA of \"%s\" on "
                      "physical volume%s is consistent\n",
