@@ -1,11 +1,12 @@
 /*
  * tools/vgcreate.c
  *
- * Copyright (C) 1997 - 1999  Heinz Mauelshagen, Germany
+ * Copyright (C) 1997 - 2000  Heinz Mauelshagen, Sistina Software
  *
  * March-June 1997
  * May,June,September 1998
- * January,February 1999
+ * January,February,October 1999
+ * February 2000
  *
  * LVM is free software; you can redistribute it and/or modify
  * it under the terms of the GNU General Public License as published by
@@ -18,7 +19,7 @@
  * GNU General Public License for more details.
  * 
  * You should have received a copy of the GNU General Public License
- * along with GNU CC; see the file COPYING.  If not, write to
+ * along with LVM; see the file COPYING.  If not, write to
  * the Free Software Foundation, 59 Temple Place - Suite 330,
  * Boston, MA 02111-1307, USA. 
  *
@@ -41,6 +42,8 @@
  *    05/09/1998 - corrected some messages
  *    26/01/1999 - made volume group directory prefix a preprocessor option
  *    06/02/1999 - fixed lvm_check_number() usage
+ *    06/10/1999 - implemented support for long options
+ *    15/02/2000 - use lvm_error()
  *
  */
 
@@ -57,6 +60,7 @@ int main ( int argc, char **argv) {
    int count_sav = 0;
    int np = 0;
    int opt_A = 1;
+   int opt_A_set = 0;
    int opt_l = 0;
    int opt_p = 0;
    int opt_s = 0;
@@ -79,10 +83,22 @@ int main ( int argc, char **argv) {
    pv_t **pv = NULL;
    pv_t **pvp = NULL;
 #ifdef DEBUG
-   char *options = "A:de:l:p:h?s:v";
+   char *options = "A:dh?l:p:s:v";
 #else
-   char *options = "A:e:l:p:h?s:v";
+   char *options = "A:h?l:p:s:v";
 #endif
+   struct option long_options[] = {
+      { "autobackup",         required_argument, NULL, 'A'},
+#ifdef DEBUG
+      { "debug",              no_argument,       NULL, 'd'},
+#endif
+      { "help",               no_argument,       NULL, 'h'},
+      { "maxlogicalvolumes",  required_argument, NULL, 'l'},
+      { "maxphysicalvolumes", required_argument, NULL, 'p'},
+      { "physicalextentsize", required_argument, NULL, 's'},
+      { "verbose",            no_argument,       NULL, 'v'},
+      { NULL, 0, NULL, 0}
+   };
    char *pv_name = NULL;
    char **vg_name_ptr = NULL;
 
@@ -91,9 +107,11 @@ int main ( int argc, char **argv) {
    SUSER_CHECK;
    LVMTAB_CHECK;
 
-   while ( ( c = getopt ( argc, argv, options)) != EOF) {
+   while ( ( c = getopt_long ( argc, argv, options,
+                               long_options, NULL)) != EOF) {
       switch ( c) {
          case 'A':
+            opt_A_set++;
             if ( opt_A > 1) {
                fprintf ( stderr, "%s -- A option already given\n\n", cmd);
                return LVM_EINVALID_CMD_LINE;
@@ -119,7 +137,7 @@ int main ( int argc, char **argv) {
 
          case 'h':
          case '?':
-            printf ( "\n%s\n\n%s -- Volume Group Create\n\n"
+            printf ( "\n%s  (IOP %d)\n\n%s -- Volume Group Create\n\n"
                      "Synopsis:\n"
                      "---------\n\n"
                      "%s\n"
@@ -127,14 +145,15 @@ int main ( int argc, char **argv) {
 #ifdef DEBUG
                      "\t[-d]\n"
 #endif
-                     "\t[-l MaxLogicalVolumes]\n"
-                     "\t[-p MaxPhysicalVolumes]\n"
-                     "\t[-h/-?]\n"
-                     "\t[-s PhysicalExtentSize[kKmMgGtT]]\n"
-                     "\t[-v]\n"
+                     "\t[-l/--maxlogicalvolumes MaxLogicalVolumes]\n"
+                     "\t[-p/--maxphysicalvolumes MaxPhysicalVolumes]\n"
+                     "\t[-h/-?/--help]\n"
+                     "\t[-s/--physicalextentsize "
+                     "PhysicalExtentSize[kKmMgGtT]]\n"
+                     "\t[-v/--verbose]\n"
                      "\tVolumeGroupName\n"
                      "\tPhysicalDevicePath [PhysicalDevicePath...]\n\n",
-                     lvm_version, cmd, cmd);
+                     lvm_version, LVM_LIB_IOP_VERSION,  cmd, cmd);
             return 0;
             break;
 
@@ -172,7 +191,7 @@ int main ( int argc, char **argv) {
                return LVM_EINVALID_CMD_LINE;
             }
             if ( ( pe_size = lvm_check_number ( optarg, TRUE)) > 0) {
-               pe_size *= 2;
+               pe_size = ( pe_size * 1024) / SECTOR_SIZE;
                /* if no unit has been given --> asume megabyte */
                if ( isdigit ( optarg[strlen ( optarg)-1])) pe_size *= 1024;
                opt_s++;
@@ -207,6 +226,8 @@ int main ( int argc, char **argv) {
    }
 
    CMD_MINUS_CHK;
+   CMD_CHECK_OPT_A_SET;
+
 
    if ( optind == argc) {
       fprintf ( stderr, "%s -- please enter a volume group name"
@@ -229,6 +250,16 @@ int main ( int argc, char **argv) {
       return LVM_EVGCREATE_VG_CHECK_NAME;
    } 
 
+   /* does /dev/VolumeGroupName already exist? */
+   if ( opt_v > 0) printf ( "%s -- checking volume group "
+                            "directory existence\n", cmd);
+   if ( ( ret = vg_check_dir ( vg_name)) == TRUE) {
+      fprintf ( stderr, "%s -- volume group directory or file already exists\n"
+                        "%s -- please choose a different name \n\n",
+                        cmd, cmd);
+      return LVM_EVGCREATE_VG_CHECK_DIR;
+   } 
+
    LVM_LOCK ( 0);
    LVM_CHECK_IOP;
 
@@ -256,8 +287,8 @@ int main ( int argc, char **argv) {
    if ( opt_v > 0) printf ( "%s -- reading all physical volume data from "
                             "disks\n", cmd);
    if ( ( ret = pv_read_all_pv ( &pv, FALSE)) < 0) {
-	  fprintf ( stderr, "%s -- ERROR %d reading physical volumes\n\n",
-                cmd, ret);
+	  fprintf ( stderr, "%s -- ERROR \"%s\" reading physical volumes\n\n",
+                            cmd, lvm_error ( ret));
 	  return LVM_EVGCREATE_PV_READ_ALL_PV;
    }
 
@@ -283,9 +314,9 @@ int main ( int argc, char **argv) {
                                      cmd, pv_name,
                                      cmd, pv_name);
 	    if ( ( size = pv_get_size ( pv[p]->pv_name, NULL)) <= 0) {
-	       fprintf ( stderr, "%s -- ERROR %d getting size of "
+	       fprintf ( stderr, "%s -- ERROR \"%s\" getting size of "
                                  "physical volume \"%s\"\n\n",
-                                 cmd, size, pv_name);
+                                 cmd, lvm_error ( size), pv_name);
                return LVM_EVGCREATE_PV_GET_SIZE;
             }
             if ( opt_v > 0) printf ( "%s -- size of physical volume \"%s\" "
@@ -386,8 +417,8 @@ int main ( int argc, char **argv) {
             LVM_VG_DISK_SIZE +
             max_pv * NAME_LEN +
             max_lv * sizeof ( lv_t));
-   if ( size / SECTOR_SIZE > min_pv_size / 10) {
-      fprintf ( stderr, "%s -- more than 10%% [%d KB] of physical volume "
+   if ( size / SECTOR_SIZE > min_pv_size / 5) {
+      fprintf ( stderr, "%s -- more than 20%% [%d KB] of physical volume "
                         "%s with %u KB would be used\n\n",
                         cmd,
                         size,
@@ -412,9 +443,9 @@ int main ( int argc, char **argv) {
                                                  SHORT)));
                            free ( dummy); dummy = NULL;
       } else {
-         fprintf ( stderr, "%s -- ERROR %d setting volume group \"%s\" up "
+         fprintf ( stderr, "%s -- ERROR \"%s\" setting volume group \"%s\" up "
                            "for create\n\n",
-                           cmd, ret, vg_name);
+                           cmd, lvm_error ( ret), vg_name);
       }
       return LVM_EVGCREATE_VG_SETUP;
    }
@@ -434,9 +465,9 @@ int main ( int argc, char **argv) {
    if ( opt_v > 0) printf ( "%s -- storing volume group data on disk(s)\n",
                             cmd);
    if ( ( ret = vg_write_with_pv_and_lv ( &vg)) < 0) {
-      fprintf ( stderr, "%s -- ERROR %d storing data of volume group "
+      fprintf ( stderr, "%s -- ERROR \"%s\" storing data of volume group "
                         "\"%s\" on disk(s)\n\n",
-                        cmd, ret, vg_name);
+                        cmd, lvm_error ( ret), vg_name);
       return LVM_EVGCREATE_VG_WRITE;
    }
 
@@ -468,11 +499,11 @@ int main ( int argc, char **argv) {
                            "%s/group\n",
                            cmd, vg_name);
       } else {
-         fprintf ( stderr, "%s -- ERROR %d creating volume group "
+         fprintf ( stderr, "%s -- ERROR \"%s\" creating volume group "
                            "directory "
                            LVM_DIR_PREFIX
                            "%s\n",
-                           cmd, ret, vg_name);
+                           cmd, lvm_error ( ret), vg_name);
       }
    }
 
@@ -482,9 +513,9 @@ int main ( int argc, char **argv) {
                             "in kernel\n",
                             cmd, vg_name);
    if ( ( ret = vg_create ( vg_name, &vg)) < 0) {
-      fprintf ( stderr, "%s -- ERROR %d creating VGDA for volume group \"%s\" "
-                        "in kernel\n",
-                        cmd, ret, vg_name);
+      fprintf ( stderr, "%s -- ERROR \"%s\" creating VGDA for volume "
+                        "group \"%s\" in kernel\n",
+                        cmd, lvm_error ( ret), vg_name);
       if ( ret == -LVM_EVG_CREATE_REMOVE_OPEN)
          fprintf ( stderr, "%s -- LVM not in kernel?\n", cmd);
       fprintf ( stderr, "\n");
@@ -494,9 +525,9 @@ int main ( int argc, char **argv) {
    if ( opt_v > 0) printf ( "%s -- inserting volume group \"%s\" into \"%s\"\n",
                             cmd, vg_name, LVMTAB);
    if ( ( ret = lvm_tab_vg_insert ( vg_name)) < 0) {
-      fprintf ( stderr, "%s -- ERROR %d inserting volume group \"%s\" "
+      fprintf ( stderr, "%s -- ERROR \"%s\" inserting volume group \"%s\" "
                         "into \"%s\"\n\n",
-                        cmd, ret, vg_name, LVMTAB);
+                        cmd, lvm_error ( ret), vg_name, LVMTAB);
       return LVM_EVGCREATE_VG_INSERT;
    }
 
@@ -506,13 +537,18 @@ int main ( int argc, char **argv) {
                   cmd, vg_name);
          if ( ( ret = vg_cfgbackup ( vg_name, VG_BACKUP_DIR,
                                      opt_v, &vg)) < 0) {
-            fprintf ( stderr, "%s -- ERROR %d writing VG backup\n\n", cmd, ret);
+            fprintf ( stderr, "%s -- ERROR \"%s\" writing VG backup\n\n",
+                              cmd, lvm_error ( ret));
             return LVM_EVGCREATE_VG_CFGBACKUP;
          }
+      } else {
+         printf ( "%s -- WARNING: you don't have an automatic "
+                  "backup of \"%s\"\n",
+                  cmd, vg_name);
       }
    } else {
-      fprintf ( stderr, "%s -- ERROR %d writing \"%s\"\n\n",
-                        cmd, ret, LVMTAB);
+      fprintf ( stderr, "%s -- ERROR \"%s\" writing \"%s\"\n\n",
+                        cmd, lvm_error ( ret), LVMTAB);
       return LVM_EVGCREATE_VG_CFGBACKUP_LVMTAB;
    }
 
