@@ -1,11 +1,12 @@
 /*
  * tools/vgdisplay.c
  *
- * Copyright (C) 1997 - 1999  Heinz Mauelshagen, Germany
+ * Copyright (C) 1997 - 2000  Heinz Mauelshagen, Sistina Software
  *
  * March-July,November 1997
  * January,May,June,September 1998
- * February,May 1999
+ * February,May,October,November 1999
+ * February,March 2000
  *
  * LVM is free software; you can redistribute it and/or modify
  * it under the terms of the GNU General Public License as published by
@@ -37,6 +38,10 @@
  *    02/09/1998 - corrected some messages
  *    21/02/1999 - removed LVM_LOCK and LVM_UNLOCK
  *    27/05/1999 - reformated -s output
+ *    06/10/1999 - implemented support for long options
+ *    03/11/1999 - implemented option -c for colon seperated output
+ *    15/02/2000 - use lvm_error()
+ *    08/03/2000 - check for mutual exclusive -c/-s
  *
  */
 
@@ -54,6 +59,7 @@ int main ( int argc, char **argv) {
    int len = 0;
    int max_len = 0;
    int opt_A = 0;
+   int opt_c = 0;
    int opt_D = 0;
    int opt_s = 0;
    int opt_v = 0;
@@ -61,19 +67,31 @@ int main ( int argc, char **argv) {
    char *dummy1 = NULL;
    char *dummy2 = NULL;
 #ifdef DEBUG
-   char *options = "AdDh?sv";
+   char *options = "cAdDh?sv";
 #else
-   char *options = "ADh?sv";
+   char *options = "cADh?sv";
 #endif
+   struct option long_options[] = {
+      { "activevolumegroups", no_argument, NULL, 'A'},
+      { "colon",              no_argument, NULL, 'c'},
+#ifdef DEBUG
+      { "debug",              no_argument, NULL, 'd'},
+#endif
+      { "disk",               no_argument, NULL, 'D'},
+      { "help",               no_argument, NULL, 'h'},
+      { "short",              no_argument, NULL, 's'},
+      { "verbose",            no_argument, NULL, 'v'},
+      { NULL, 0, NULL, 0}
+   };
    char *vg_name = NULL;
    vg_t *vg = NULL;
 
    cmd = basename ( argv[0]);
 
    SUSER_CHECK;
-   LVMTAB_CHECK;
 
-   while ( ( c = getopt ( argc, argv, options)) != EOF) {
+   while ( ( c = getopt_long ( argc, argv, options,
+                               long_options, NULL)) != EOF) {
       switch ( c) {
          case 'A':
             if ( opt_A > 0) {
@@ -83,6 +101,14 @@ int main ( int argc, char **argv) {
             opt_A++;
             break;
 
+         case 'c':
+            if ( opt_c > 0) {
+               fprintf ( stderr, "%s -- c option already given\n\n", cmd);
+               return LVM_EINVALID_CMD_LINE;
+            }
+            opt_c++;
+            break;
+
 #ifdef DEBUG
          case 'd':
             if ( opt_d > 0) {
@@ -103,20 +129,20 @@ int main ( int argc, char **argv) {
 
          case 'h':
          case '?':
-            printf ( "\n%s\n\n%s -- Volume Group Display\n\n"
+            printf ( "\n%s  (IOP %d)\n\n%s -- Volume Group Display\n\n"
                      "Synopsis:\n"
                      "---------\n\n"
                      "%s\n"
-                     "\t[-A]\n"
+                     "\t[-A/--activevolumegroups]\n"
+                     "\t[-c/--colon / -s/--short]\n"
 #ifdef DEBUG
-                     "\t[-d]\n"
+                     "\t[-d/--debug]\n"
 #endif
-                     "\t[-D]\n"
-                     "\t[-h/?]\n"
-                     "\t[-s]\n"
-                     "\t[-v[v]]\n"
+                     "\t[-D/--disk]\n"
+                     "\t[-h/-?/--help]\n"
+                     "\t[-v[v]/--verbose [--verbose]]\n"
                      "\t[VolumeGroupName...]\n\n",
-                     lvm_version, cmd, cmd);
+                     lvm_version, LVM_LIB_IOP_VERSION,  cmd, cmd);
             return 0;
             break;
 
@@ -144,14 +170,25 @@ int main ( int argc, char **argv) {
    }
 
    CMD_MINUS_CHK;
+   LVMTAB_CHECK;
    LVM_CHECK_IOP;
 
-   if ( opt_s > 0 && opt_v > 0) {
-      fprintf ( stderr, "%s -- don't give options s and v together\n\n", cmd);
+   if ( opt_c + opt_v > 1) {
+      fprintf ( stderr, "%s -- option v is not allowed with option c\n\n", cmd);
       return LVM_EINVALID_CMD_LINE;
    }
 
-   if ( opt_A > 0 && opt_D > 0) {
+   if ( opt_s + opt_v > 1) {
+      fprintf ( stderr, "%s -- option v is not allowed with option s\n\n", cmd);
+      return LVM_EINVALID_CMD_LINE;
+   }
+
+   if ( opt_c + opt_s > 1) {
+      fprintf ( stderr, "%s -- option c is not allowed with option s\n\n", cmd);
+      return LVM_EINVALID_CMD_LINE;
+   }
+
+   if ( opt_A + opt_D > 1) {
       fprintf ( stderr, "%s -- option D is not allowed with option A\n\n", cmd);
       return LVM_EINVALID_CMD_LINE;
    }
@@ -187,15 +224,13 @@ int main ( int argc, char **argv) {
    }
 
    /* figure out longest volume group name */
-   c = optind;
-   for ( ; optind < argc; optind++) {
+   for ( c = optind; optind < argc; optind++) {
       len = strlen ( argv[optind]);
       if ( len > max_len) max_len = len;
    }
 
    /* loop through all volume groups */
-   optind = c;
-   for ( ; optind < argc; optind++) {
+   for ( optind = c; optind < argc; optind++) {
       vg_name = argv[optind];
 
       if ( opt_v > 1) printf ( "%s -- checking volume group name \"%s\"\n",
@@ -227,11 +262,11 @@ int main ( int argc, char **argv) {
                vg_status ( vg_name, &vg);
                if ( vg->lv_open > 0) printf ( "/used");
                else                  printf ( "/idle");
-               printf ( ") %-8s [%-8s used / ",
+               printf ( ") %-9s [%-9s used / ",
                         ( dummy1 = lvm_show_size ( vg->pe_total *
                                                    vg->pe_size / 2,
                                                    SHORT)),
-                        ( dummy1 = lvm_show_size ( vg->pe_allocated *
+                        ( dummy2 = lvm_show_size ( vg->pe_allocated *
                                                   vg->pe_size / 2,
                                                   SHORT)));
                free ( dummy1); dummy1 = NULL;
@@ -251,11 +286,10 @@ int main ( int argc, char **argv) {
                if ( opt_v > 1) printf ( "%s -- getting VGDA of \"%s\" from "
                                         "kernel\n", cmd, vg_name);
                if ( opt_v > 0) {
-                  if ( ( ret = vg_status_with_pv_and_lv ( vg_name, &vg,
-                                                          TRUE)) < 0) {
+                  if ( ( ret = vg_status_with_pv_and_lv ( vg_name, &vg)) < 0) {
                      fprintf ( stderr,
-                               "%s -- ERROR %d getting status of \"%s\"\n",
-                                cmd, ret, vg_name);
+                               "%s -- ERROR \"%s\" getting status of \"%s\"\n",
+                                cmd, lvm_error ( ret), vg_name);
                      continue;
                   }
                   vg_show ( vg);
@@ -266,8 +300,10 @@ int main ( int argc, char **argv) {
                   printf ( "\n");
                } else {
                   vg_status ( vg_name, &vg);
-                  vg_show ( vg);
-                  printf ( "\n");
+                  if ( opt_c == 0) {
+                     vg_show ( vg);
+                     printf ( "\n");
+                  } else vg_show_colon ( vg);
                }
             } else {
                printf ( "%s -- \"%s\" is NOT active; try -D\n",
@@ -288,9 +324,9 @@ int main ( int argc, char **argv) {
                                 cmd, vg_name);
                   } else {
                      fprintf ( stderr,
-                               "%s -- ERROR %d getting data of \"%s\" from "
+                               "%s -- ERROR \"%s\" getting data of \"%s\" from "
                                "disk(s)\n",
-                               cmd, ret, vg_name);
+                               cmd, lvm_error ( ret), vg_name);
                   }
                   continue;
                }
@@ -314,15 +350,17 @@ int main ( int argc, char **argv) {
                                cmd, vg_name);
                   } else {
                      fprintf ( stderr,
-                               "%s -- ERROR %d reading data of \"%s\" "
+                               "%s -- ERROR \"%s\" reading data of \"%s\" "
                                "from disk(s)\n",
-                               cmd, ret, vg_name);
+                               cmd, lvm_error ( ret), vg_name);
                   }
                   continue;
                }
                else {
-                  vg_show ( vg);
-                  printf ( "\n");
+                  if ( opt_c == 0) {
+                     vg_show ( vg);
+                     printf ( "\n");
+                  } else vg_show_colon ( vg);
                }
             }
          }
@@ -334,6 +372,10 @@ int main ( int argc, char **argv) {
          fprintf ( stderr, "%s -- ERROR: not all physical volumes of "
                            "volume group \"%s\" online\n", cmd, vg_name);
          exit_code = LVM_EVGDISPLAY_PV_COUNT;
+      } else if ( ret == -LVM_EVG_CFGRESTORE_READ) {
+         fprintf ( stderr, "%s -- ERROR reading lvmtab of \"%s\"\n",
+                           cmd, vg_name);
+         exit_code = LVM_EVGDISPLAY_VG_CFGRESTORE;
       } else {
          fprintf ( stderr, "%s -- volume group \"%s\" not found\n",
                            cmd, vg_name);
