@@ -1,10 +1,12 @@
 /*
  * tools/vgexport.c
  *
- * Copyright (C) 1997 - 1999  Heinz Mauelshagen, Germany
+ * Copyright (C) 1997 - 2000  Heinz Mauelshagen, Sistina Software
  *
  * May 1997
  * May,June,September 1998
+ * October 1999
+ * February 2000
  *
  * LVM is free software; you can redistribute it and/or modify
  * it under the terms of the GNU General Public License as published by
@@ -37,6 +39,8 @@
  *               - used lvm_tab_vg_check_exist() instead of vg_check_exist()
  *               - checked for volume group activity first for
  *                 better performance
+ *    06/10/1999 - implemented support for long options
+ *    15/02/2000 - use lvm_error()
  *
  */
 
@@ -60,6 +64,15 @@ int main ( int argc, char **argv) {
 #else
    char *options = "ah?v";
 #endif
+   struct option long_options[] = {
+      { "all",     no_argument, NULL, 'a'},
+#ifdef DEBUG
+      { "debug",   no_argument, NULL, 'd'},
+#endif
+      { "help",    no_argument, NULL, 'h'},
+      { "verbose", no_argument, NULL, 'v'},
+      { NULL,      0,           NULL, 0},
+   };
    char vg_name_exp[NAME_LEN] = { 0, };
    vg_t *vg = NULL;
 
@@ -68,7 +81,8 @@ int main ( int argc, char **argv) {
    SUSER_CHECK;
    LVMTAB_CHECK;
 
-   while ( ( c = getopt ( argc, argv, options)) != EOF) {
+   while ( ( c = getopt_long ( argc, argv, options,
+                               long_options, NULL)) != EOF) {
       switch ( c) {
 
          case 'a':
@@ -91,18 +105,18 @@ int main ( int argc, char **argv) {
 
          case 'h':
          case '?':
-            printf ( "\n%s\n\n%s -- Volume Group Export\n\n"
+            printf ( "\n%s  (IOP %d)\n\n%s -- Volume Group Export\n\n"
                      "Synopsis:\n"
                      "---------\n\n"
                      "%s\n"
-                     "\t[-a]\n"
+                     "\t[-a/--all]\n"
 #ifdef DEBUG
-                     "\t[-d]\n"
+                     "\t[-d/--debug]\n"
 #endif
-                     "\t[-h/?]\n"
-                     "\t[-v]\n"
+                     "\t[-h/-?/--help]\n"
+                     "\t[-v/--verbose]\n"
                      "\tVolumeGroupName [VolumeGroupName...]\n\n",
-                     lvm_version, cmd, cmd);
+                     lvm_version, LVM_LIB_IOP_VERSION,  cmd, cmd);
             return 0;
             break;
 
@@ -177,9 +191,9 @@ int main ( int argc, char **argv) {
       if ( opt_v > 0) printf ( "%s -- reading data of volume group \"%s\" "
                                "from disk(s)\n", cmd, vg_name);
       if ( ( ret = lvm_tab_vg_read_with_pv_and_lv ( vg_name, &vg)) != 0) {
-         fprintf ( stderr, "%s -- ERROR %d reading data of volume "
+         fprintf ( stderr, "%s -- ERROR \"%s\" reading data of volume "
                            "group \"%s\"\n\n",
-                           cmd, ret, vg_name);
+                           cmd, lvm_error ( ret), vg_name);
          continue;
       }
 
@@ -194,9 +208,9 @@ int main ( int argc, char **argv) {
       if ( opt_v > 0) printf ( "%s -- checking volume group \"%s\" "
                                "consistency\n", cmd, vg_name);
       if ( ( ret = vg_check_consistency_with_pv_and_lv ( vg)) != 0) {
-         fprintf ( stderr, "%s -- ERROR %d: volume group \"%s\" is "
+         fprintf ( stderr, "%s -- ERROR \"%s\" volume group \"%s\" is "
                            "inconsistent\n\n",
-                           cmd, ret, vg_name);
+                           cmd, lvm_error ( ret), vg_name);
          continue;
       }
 
@@ -223,8 +237,9 @@ int main ( int argc, char **argv) {
       lvm_dont_interrupt ( 0);
 
       if ( ( ret = vg_write_with_pv_and_lv ( vg)) < 0) {
-         fprintf ( stderr, "%s -- ERROR %d storing volume group "
-                           "data on disk(s)\n", cmd, ret);
+         fprintf ( stderr, "%s -- ERROR \"%s\" storing volume group "
+                           "data on disk(s)\n",
+                           cmd, lvm_error ( ret));
       }
 
       if ( opt_v > 0) printf ( "%s -- removing special files of volume "
@@ -233,15 +248,15 @@ int main ( int argc, char **argv) {
       vg_remove_dir_and_group_and_nodes ( vg_name);
    
       if ( ( ret = lvm_tab_vg_remove ( vg_name)) < 0) {
-         fprintf ( stderr, "%s -- ERROR %d removing volume group \"%s\" "
+         fprintf ( stderr, "%s -- ERROR \"%s\" removing volume group \"%s\" "
                            "from \"%s\"\n",
-                           cmd, ret, vg_name, LVMTAB);
+                           cmd, lvm_error ( ret), vg_name, LVMTAB);
          continue;
       }
 
       lvm_interrupt ();
 
-      printf ( "%s -- volume group \"%s\" %s  sucessfully exported\n",
+      printf ( "%s -- volume group \"%s\"%s sucessfully exported\n",
                cmd, vg_name, ret < 0 ? " NOT" : "");
       if ( ret < 0) continue;
 
