@@ -1,11 +1,12 @@
 /*
  * tools/vgimport.c
  *
- * Copyright (C) 1997 - 1999  Heinz Mauelshagen, Germany
+ * Copyright (C) 1997 - 2000  Heinz Mauelshagen, Sistina Software
  *
  * May 1997
  * April-June,September 1998
- * February,May 1999
+ * February,May,October 1999
+ * February 2000
  *
  * LVM is free software; you can redistribute it and/or modify
  * it under the terms of the GNU General Public License as published by
@@ -39,6 +40,8 @@
  *    05/09/1998 - corrected some messages
  *    16/02/1999 - changed to new lvm_tab_get_free_blk_dev()
  *    20/05/1999 - fixed MAX_LV check
+ *    06/10/1999 - implemented support for long options
+ *    15/02/2000 - use lvm_error()
  *
  */
 
@@ -65,10 +68,20 @@ int main ( int argc, char **argv) {
    char *vg_name = NULL;
    char vg_name_this[NAME_LEN] = { 0, };
 #ifdef DEBUG
-   char *options = "dh?v";
+   char *options = "dfh?v";
 #else
-   char *options = "h?v";
+   char *options = "fh?v";
 #endif
+   struct option long_options[] = {
+#ifdef DEBUG
+      { "debug",   no_argument, NULL, 'd'},
+#endif
+      { "force",   no_argument, NULL, 'f'},
+      { "help",    no_argument, NULL, 'h'},
+      { "verbose", no_argument, NULL, 'v'},
+      { NULL,      0,           NULL, 0},
+   };
+   int opt_f = 0;
    int opt_v = 0;
    kdev_t *blk_dev_free = NULL;
    pv_t *pv = NULL;
@@ -80,7 +93,8 @@ int main ( int argc, char **argv) {
    SUSER_CHECK;
    LVMTAB_CHECK;
 
-   while ( ( c = getopt ( argc, argv, options)) != EOF) {
+   while ( ( c = getopt_long ( argc, argv, options,
+                               long_options, NULL)) != EOF) {
       switch ( c) {
 #ifdef DEBUG
          case 'd':
@@ -92,20 +106,29 @@ int main ( int argc, char **argv) {
             break;
 #endif
 
+         case 'f':
+            if ( opt_f > 0) {
+               fprintf ( stderr, "%s -- f option already given\n\n", cmd);
+               return LVM_EINVALID_CMD_LINE;
+            }
+            opt_f++;
+            break;
+
          case 'h':
          case '?':
-            printf ( "\n%s\n\n%s -- Volume Group Import\n\n"
+            printf ( "\n%s  (IOP %d)\n\n%s -- Volume Group Import\n\n"
                      "Synopsis:\n"
                      "---------\n\n"
                      "%s\n"
 #ifdef DEBUG
-                     "\t[-d]\n"
+                     "\t[-d/--debug]\n"
 #endif
-                     "\t[-h/?]\n"
-                     "\t[-v]\n"
+                     "\t[-f/--force]\n"
+                     "\t[-h/-?/--help]\n"
+                     "\t[-v/--verbose]\n"
                      "\tVolumeGroupName PhysicalVolumePath "
                      "[PhysicalVolumePath...]\n\n",
-                     lvm_version, cmd, cmd);
+                     lvm_version, LVM_LIB_IOP_VERSION,  cmd, cmd);
             return 0;
             break;
 
@@ -155,7 +178,8 @@ int main ( int argc, char **argv) {
    }
 
    /* get the physical volume data */
-   if ( opt_v > 0) printf ( "%s -- trying to read physical volumes\n", cmd);
+   if ( opt_v > 0) printf ( "%s -- trying to read physical volume%s\n",
+                            cmd, argc - optind > 1 ? "s" : "");
    p = 0;
    for ( ; optind < argc; optind++) {
       pv_name = argv[optind];
@@ -183,16 +207,17 @@ int main ( int argc, char **argv) {
                                "from disk\n", cmd, pv_name);
       if ( ( ret = pv_read ( pv_name, &pv, NULL)) < 0 &&
            ret != -LVM_EPV_READ_PV_EXPORTED) {
-         fprintf ( stderr, "%s -- ERROR %d reading data of physical "
+         fprintf ( stderr, "%s -- ERROR \"%s\" reading data of physical "
                            "volume \"%s\"\n\n",
-                           cmd, ret, pv_name);
+                           cmd, lvm_error ( ret), pv_name);
          return LVM_EVGIMPORT_PV_READ;
       }
 
       if ( opt_v > 0) printf ( "%s -- checking for exported "
                                "physical volume \"%s\"\n",
                                cmd, pv_name);
-      if ( system_id_check_exported ( pv->system_id) == FALSE) {
+      if ( opt_f == 0 &&
+           system_id_check_exported ( pv->system_id) == FALSE) {
          fprintf ( stderr, "%s -- ERROR: physical volume \"%s\" doesn't belong "
                            "to an exported volume group\n\n",
                            cmd, pv_name);
@@ -217,9 +242,9 @@ int main ( int argc, char **argv) {
       if ( opt_v > 0) printf ( "%s -- checking consistency of physical "
                                "volume \"%s\"\n", cmd, pv_name);
       if ( ( ret = pv_check_consistency ( pv_this[p])) < 0) {
-         fprintf ( stderr, "%s -- ERROR %d checking physical "
+         fprintf ( stderr, "%s -- ERROR \"%s\" checking physical "
                            "volume consistency of \"%s\"\n",
-                   cmd, ret, pv_name);
+                           cmd, lvm_error ( ret), pv_name);
          continue;
       }
 
@@ -288,9 +313,9 @@ int main ( int argc, char **argv) {
    if ( opt_v > 0) printf ( "%s -- reading exported volume group data "
                             " of \"%s\" from disk(s)\n", cmd, vg_name_this);
    if ( ( ret = vg_read_with_pv_and_lv ( pv_this[0]->vg_name, &vg)) < 0) {
-      fprintf ( stderr, "%s -- ERROR %d reading data of "
-                        "exported volume group \"%s\"\n\n",
-                        cmd, ret, vg_name_this);
+      fprintf ( stderr, "%s -- ERROR \"%s\" reading data of "
+                        "volume group \"%s\"\n\n",
+                        cmd, lvm_error ( ret), vg_name_this);
       return LVM_EVGIMPORT_VG_READ;
    }
 
@@ -319,10 +344,11 @@ int main ( int argc, char **argv) {
                             "\"%s\" consistency\n", cmd, vg->vg_name);
    if ( ( ret = vg_check_consistency_with_pv_and_lv ( vg)) < 0 &&
         ret != -LVM_EVG_CHECK_CONSISTENCY_VG_STATUS) {
-      fprintf ( stderr, "%s -- ERROR %d: exported volume group \"%s\" "
+      fprintf ( stderr, "%s -- ERROR \"%s\" exported volume group \"%s\" "
                         "is inconsistent\n"
                         "%s -- it's system id is \"%s\"\n\n",
-                        cmd, ret, vg_name_this, cmd, vg->pv[0]->system_id);
+                        cmd, lvm_error ( ret), vg_name_this,
+                        cmd, vg->pv[0]->system_id);
       return LVM_EVGIMPORT_VG_CHECK_CONSISTENCY;
    }
 
@@ -337,8 +363,8 @@ int main ( int argc, char **argv) {
    strcpy ( vg->vg_name, vg_name);
 
    if ( opt_v > 0) printf ( "%s -- importing physical volumes of volume "
-                            "group \"%s\"\n",
-                            cmd, vg->vg_name);
+                            "group \"%s\"%s\n",
+                            cmd, vg->vg_name, opt_f > 1 ? "(forced)" : "");
    for ( p = 0; vg->pv[p] != NULL; p++) {
       system_id_set_imported ( vg->pv[p]->system_id);
       strcpy ( vg->pv[p]->vg_name, vg_name);
@@ -360,8 +386,9 @@ int main ( int argc, char **argv) {
                             "volume group \"%s\"\n\n",
                             cmd, vg_name);
    if ( ( blk_dev_free_count = lvm_tab_get_free_blk_dev ( &blk_dev_free)) < 0) {
-      fprintf ( stderr, "%s -- ERROR %d getting free block device numbers\n\n",
-                        cmd, blk_dev_free_count);
+      fprintf ( stderr, "%s -- ERROR \"%s\" getting free block device "
+                        "numbers\n\n",
+                        cmd, lvm_error ( blk_dev_free_count));
       return LVM_EVGIMPORT_NO_DEV;
    }
 
@@ -391,9 +418,9 @@ int main ( int argc, char **argv) {
                             "\"%s\" " "on disk(s)\n",
                             cmd, vg_name);
    if ( ( ret = vg_write_with_pv_and_lv ( vg)) < 0) {
-      fprintf ( stderr, "%s -- ERROR %d storing data of imported volume "
+      fprintf ( stderr, "%s -- ERROR \"%s\" storing data of imported volume "
                         "\"%s\" on disks\n\n",
-                        cmd, ret, vg_name);
+                        cmd, lvm_error ( ret), vg_name);
       return LVM_EVGIMPORT_VG_WRITE;
    }
 
@@ -416,9 +443,9 @@ int main ( int argc, char **argv) {
                             "\"%s\" into \"%s\"\n",
                             cmd, vg_name, LVMTAB);
    if ( ( ret = lvm_tab_vg_insert ( vg_name)) < 0) {
-      fprintf ( stderr, "%s -- ERROR %d inserting volume group \"%s\" "
+      fprintf ( stderr, "%s -- ERROR \"%s\" inserting volume group \"%s\" "
                        "into \"%s\"\n\n",
-                       cmd, ret, vg_name, LVMTAB);
+                       cmd, lvm_error ( ret), vg_name, LVMTAB);
       return LVM_EVGIMPORT_VG_INSERT;
    }
 
@@ -427,11 +454,13 @@ int main ( int argc, char **argv) {
                cmd, vg_name);
       if ( ( ret = vg_cfgbackup ( vg_name, VG_BACKUP_DIR,
                                   opt_v, vg)) < 0) {
-         fprintf ( stderr, "%s -- ERROR %d writing VG backup\n\n", cmd, ret);
+         fprintf ( stderr, "%s -- ERROR \"%s\" writing VG backup\n\n",
+                           cmd, lvm_error ( ret));
          return LVM_EVGIMPORT_VG_CFGBACKUP;
       }
    } else {
-      fprintf ( stderr, "%s -- ERROR %d writing lvmtab\n\n", cmd, ret);
+      fprintf ( stderr, "%s -- ERROR \"%s\" writing lvmtab\n\n",
+                        cmd, lvm_error ( ret));
       return LVM_EVGIMPORT_VG_CFGBACKUP_LVMTAB;
    }
 
