@@ -1,12 +1,13 @@
 /*
  * tools/vgscan.c
  *
- * Copyright (C) 1997 - 1999  Heinz Mauelshagen, Germany
+ * Copyright (C) 1997 - 2000  Heinz Mauelshagen, Sistina Software
  *
  * Oktober  1997
  * January  1998
  * May-July,September 1998
- * May 1999
+ * May,July,October,November 1999
+ * February,July 2000
  *
  * LVM is free software; you can redistribute it and/or modify
  * it under the terms of the GNU General Public License as published by
@@ -37,6 +38,13 @@
  *    19/05/1999 - major rewrite to correct multiple assigned volume
  *                 group numbers and block device specials
  *               - avoided option A
+ *    04/08/1999 - support for snapshot logical volumes
+ *    06/10/1999 - implemented support for long options
+ *    30/11/1999 - delete special files and directory before
+ *                 trying to create them
+ *    12/02/2000 - use lvm_remove_recursive() instead of system ( "rm -fr ...
+ *    15/02/2000 - use lvm_error()
+ *    05/07/2000 - avoided resetting of snapshots
  *
  */
 
@@ -49,10 +57,10 @@
                             cmd, vg_name_ptr[i]); \
    if ( ( ret = vg_read_with_pv_and_lv ( vg_name_ptr[i], \
                                          &vg)) < 0) { \
-      fprintf ( stderr, "%s -- ERROR %d: can't get data of " \
+      fprintf ( stderr, "%s -- ERROR \"%s\" can't get data of " \
                         "volume group \"%s\" from physical " \
                         "volume(s)\n", \
-                        cmd, ret, vg_name_ptr[i]); \
+                        cmd, lvm_error ( ret), vg_name_ptr[i]); \
       continue; \
    } \
 }
@@ -62,9 +70,9 @@
                             cmd, vg_name_ptr[i]); \
    if ( ( ret = lvm_tab_vg_insert ( vg_name_ptr[i])) < 0 && \
         ret != -LVM_ELVM_TAB_VG_INSERT_VG_EXISTS) { \
-      fprintf ( stderr, "%s -- ERROR %d inserting volume group " \
+      fprintf ( stderr, "%s -- ERROR \"%s\" inserting volume group " \
                         "\"%s\" into \"%s\"\n", \
-                        cmd, ret, vg_name_ptr[i], LVMTAB); \
+                        cmd, lvm_error ( ret), vg_name_ptr[i], LVMTAB); \
       return LVM_EVGSCAN_VG_INSERT; \
    } \
 }
@@ -80,19 +88,25 @@
                         "of volume group \"%s\"\n", \
                         cmd, vg_name_ptr[i]); \
       if ( ( ret = lvm_tab_vg_remove ( vg_name_ptr[i])) < 0) { \
-         fprintf ( stderr, "%s -- ERROR %d removing volume group " \
+         fprintf ( stderr, "%s -- ERROR \"%s\" removing volume group " \
                            "\"%s\" from \"%s\"\n", \
-                           cmd, ret, vg_name_ptr[i], LVMTAB); \
+                           cmd, lvm_error ( ret), vg_name_ptr[i], LVMTAB); \
       } \
       continue; \
    } \
-   if ( opt_v > 0) printf ( "%s -- creating volume group directory " \
-                            "and special files for \"%s\"\n", \
+   if ( opt_v > 0) printf ( "%s -- removing special files and directory " \
+                            "for volume group \"%s\"\n", \
                             cmd, vg_name_ptr[i]); \
+   if ( ( ret = vg_remove_dir_and_group_and_nodes ( vg->vg_name)) < 0) { \
+      fprintf ( stderr, "%s -- ERROR \"%s\" removing volume group " \
+                        "directory and special files\n", \
+                        cmd, lvm_error ( ret)); \
+   } \
    if ( ( ret = vg_create_dir_and_group_and_nodes ( vg, \
                                                     opt_v)) < 0) { \
-      fprintf ( stderr, "%s -- ERROR %d creating volume group " \
-                        "directory and special files\n", cmd, ret); \
+      fprintf ( stderr, "%s -- ERROR \"%s\" creating volume group " \
+                        "directory and special files\n", \
+                        cmd, lvm_error ( ret)); \
       continue; \
    } \
 }
@@ -119,7 +133,15 @@ int main ( int argc, char **argv)
 #else
    char *options = "A:h?v";
 #endif
-   char command[NAME_LEN] = { 0, };
+   struct option long_options[] = {
+      { "autobackup", required_argument, NULL, 'A'},
+#ifdef DEBUG
+      { "debug",      no_argument,       NULL, 'd'},
+#endif
+      { "help",       no_argument,       NULL, 'h'},
+      { "verbose",    no_argument,       NULL, 'v'},
+      { NULL,         0,                 NULL, 0},
+   };
    char **vg_name_ptr = NULL;
    kdev_t *blk_dev_free = NULL;
    vg_t *vg = NULL;
@@ -129,7 +151,8 @@ int main ( int argc, char **argv)
 
    SUSER_CHECK;
 
-   while ( ( c = getopt ( argc, argv, options)) != EOF) {
+   while ( ( c = getopt_long ( argc, argv, options,
+                               long_options, NULL)) != EOF) {
       switch ( c) {
 #ifdef DEBUG
          case 'd':
@@ -143,16 +166,16 @@ int main ( int argc, char **argv)
 
          case 'h':
          case '?':
-            printf ( "\n%s\n\n%s -- Volume Group Scan\n\n"
+            printf ( "\n%s  (IOP %d)\n\n%s -- Volume Group Scan\n\n"
                      "Synopsis:\n"
                      "---------\n\n"
                      "%s\n"
 #ifdef DEBUG
-                     "\t[-d]\n"
+                     "\t[-d/--debug]\n"
 #endif
-                     "\t[-h/?]\n"
-                     "\t[-v]]\n\n",
-                     lvm_version, cmd, cmd);
+                     "\t[-h/-?/--help]\n"
+                     "\t[-v/--verbose]\n\n",
+                     lvm_version, LVM_LIB_IOP_VERSION,  cmd, cmd);
             return 0;
             break;
 
@@ -184,14 +207,15 @@ int main ( int argc, char **argv)
 
    if ( opt_v > 0) printf ( "%s -- removing \"%s\" and \"%s\"\n",
                             cmd, LVMTAB, LVMTAB_DIR);
-   sprintf ( command, "rm -fr %s %s%c", LVMTAB, LVMTAB_DIR, 0);
-   system ( command);
+   unlink ( LVMTAB);
+   lvm_remove_recursive ( LVMTAB_DIR);
 
    printf ( "%s -- reading all physical volumes (this may take a while...)\n",
             cmd);
    if ( ( vg_name_ptr = vg_check_exist_all_vg ()) == NULL) {
       if ( ( ret = lvm_tab_create ()) < 0)
-         fprintf ( stderr, "%s -- ERROR %d creating lvmtab\n", cmd, ret);
+         fprintf ( stderr, "%s -- ERROR \"%s\" creating lvmtab\n",
+                           cmd, lvm_error ( ret));
       ret = LVM_EVGSCAN_NO_VG;
    } else {
       /* all active volume groups first so that i don't
@@ -253,13 +277,13 @@ int main ( int argc, char **argv)
                if ( opt_v > 0) printf ( "%s -- storing volume group data of "
                                         "%s on disk(s)\n", cmd, vg_name_ptr[i]);
                if ( ( ret = vg_write_with_pv_and_lv ( vg)) < 0) {
-                  fprintf ( stderr, "%s -- ERROR %d storing data of volume "
+                  fprintf ( stderr, "%s -- ERROR \"%s\" storing data of volume "
                                     "group \"%s\" on disk(s)\n\n",
-                                    cmd, ret, vg_name_ptr[i]);
+                                    cmd, lvm_error ( ret), vg_name_ptr[i]);
                   return LVM_EVGSCAN_VG_WRITE;
                }
             }
-            vg_free ( vg, TRUE);
+            vg_free ( vg, FALSE);
          }
       }
    }
@@ -270,13 +294,13 @@ int main ( int argc, char **argv)
    if ( ret == 0) {
       printf ( "%s -- \"%s\" and \"%s\" successfully created\n",
                cmd, LVMTAB, LVMTAB_DIR);
-      printf ( "%s -- WARNING: you may not have an actual backup of your "
+      printf ( "%s -- WARNING: you may not have an actual VGDA backup of your "
                "volume group%s\n\n", cmd, i > 1 ? "s" : "");
    } else if ( ret == LVM_EVGSCAN_NO_VG) {
       fprintf ( stderr, "%s -- no volume groups found\n\n", cmd);
    } else {
-      fprintf ( stderr, "%s -- ERROR %d creating \"%s\" and \"%s\"\n\n",
-                        cmd, ret, LVMTAB, LVMTAB_DIR);
+      fprintf ( stderr, "%s -- ERROR \"%s\" creating \"%s\" and \"%s\"\n\n",
+                        cmd, lvm_error ( ret), LVMTAB, LVMTAB_DIR);
       ret = LVM_EVGSCAN_LVMTAB;
    }
 
