@@ -1,9 +1,11 @@
 /*
  * tools/vgsplit.c
  *
- * Copyright (C) 1997 - 1999  Heinz Mauelshagen, Germany
+ * Copyright (C) 1997 - 2000  Heinz Mauelshagen, Sistina Software
  *
  * June,September 1998
+ * October 1999
+ * February 2000
  *
  * LVM is free software; you can redistribute it and/or modify
  * it under the terms of the GNU General Public License as published by
@@ -16,7 +18,7 @@
  * GNU General Public License for more details.
  * 
  * You should have received a copy of the GNU General Public License
- * along with GNU CC; see the file COPYING.  If not, write to
+ * along with LVM; see the file COPYING.  If not, write to
  * the Free Software Foundation, 59 Temple Place - Suite 330,
  * Boston, MA 02111-1307, USA. 
  *
@@ -27,6 +29,8 @@
  *
  *    27/06/1998 - changed to new vg_setup_for_split() calling convention
  *    06/09/1998 - corrected some messages
+ *    06/10/1999 - implemented support for long options
+ *    15/02/2000 - use lvm_error()
  *
  */
 
@@ -43,6 +47,7 @@ int main ( int argc, char **argv) {
    int l = 0;
    int lr = 0;
    int opt_A = 1;
+   int opt_A_set = 0;
    int opt_l = 0;
    int opt_t = 0;
    int opt_v = 0;
@@ -58,6 +63,17 @@ int main ( int argc, char **argv) {
 #else
    char *options = "A:h?ltv";
 #endif
+   struct option long_options[] = {
+      { "autobackup", required_argument, NULL, 'A'},
+#ifdef DEBUG
+      { "debug",      no_argument,       NULL, 'd'},
+#endif
+      { "help",       no_argument,       NULL, 'h'},
+      { "list",       no_argument,       NULL, 'l'},
+      { "test",       no_argument,       NULL, 't'},
+      { "verbose",    no_argument,       NULL, 'v'},
+      { NULL,         0,                 NULL, 0},
+   }; 
    char *vg_name_exist = NULL;
    char *vg_name_new = NULL;
    char *error_pv_name = NULL;
@@ -76,9 +92,11 @@ int main ( int argc, char **argv) {
    SUSER_CHECK;
    LVMTAB_CHECK;
 
-   while ( ( c = getopt ( argc, argv, options)) != EOF) {
+   while ( ( c = getopt_long ( argc, argv, options,
+                               long_options, NULL)) != EOF) {
       switch ( c) {
          case 'A':
+	    opt_A_set++;
             if ( opt_A > 1) {
                fprintf ( stderr, "%s -- A option already given\n\n", cmd);
                return LVM_EINVALID_CMD_LINE;
@@ -104,21 +122,21 @@ int main ( int argc, char **argv) {
 
          case 'h':
          case '?':
-            printf ( "\n%s\n\n%s -- Volume Group Split\n\n"
+            printf ( "\n%s  (IOP %d)\n\n%s -- Volume Group Split\n\n"
                      "Synopsis:\n"
                      "---------\n\n"
                      "%s\n"
-                     "\t[-A y/n]\n"
+                     "\t[-A/--autobackup y/n]\n"
 #ifdef DEBUG
-                     "\t[-d]\n"
+                     "\t[-d/--debug]\n"
 #endif
-                     "\t[-h/-?]\n"
-                     "\t[-l]\n"
-                     "\t[-t]\n"
-                     "\t[-v]\n"
+                     "\t[-h/-?/--help]\n"
+                     "\t[-l/--list]\n"
+                     "\t[-t/--test]\n"
+                     "\t[-v/--verbose]\n"
                      "\tExistingVolumeGroupName NewVolumeGroupName\n"
                      "\tPhysicalVolumePath [PhysicalVolumePath...]\n\n",
-                     lvm_version, cmd, cmd);
+                     lvm_version, LVM_LIB_IOP_VERSION,  cmd, cmd);
             return 0;
             break;
 
@@ -156,6 +174,8 @@ int main ( int argc, char **argv) {
    CMD_MINUS_CHK;
    LVM_CHECK_IOP;
    LVM_LOCK ( 0);
+   CMD_CHECK_OPT_A_SET;
+
 
    if ( optind > argc - 3) {
       fprintf ( stderr, "%s -- invalid command line\n\n", cmd);
@@ -201,9 +221,9 @@ int main ( int argc, char **argv) {
                             cmd, vg_name_exist, LVMTAB);
    if ( ( ret = lvm_tab_vg_read_with_pv_and_lv ( vg_name_exist,
                                                  &vg_exist)) < 0) {
-      fprintf ( stderr, "%s -- ERROR %d getting data of volume "
+      fprintf ( stderr, "%s -- ERROR \"%s\" getting data of volume "
                         "group \"%s\"\n\n",
-                        cmd, ret, vg_name_exist);
+                        cmd, lvm_error ( ret), vg_name_exist);
       return LVM_EVGSPLIT_VG_READ_EXIST;
    }
 
@@ -215,9 +235,9 @@ int main ( int argc, char **argv) {
                             cmd, vg_name_exist);
    if ( ( ret = lvm_tab_vg_read_with_pv_and_lv ( vg_name_exist,
                                                  &vg_exist_sav)) < 0) {
-      fprintf ( stderr, "%s -- ERROR %d getting data of volume "
+      fprintf ( stderr, "%s -- ERROR \"%s\" getting data of volume "
                         "group \"%s\" (saved)\n\n",
-                        cmd, ret, vg_name_exist);
+                        cmd, lvm_error ( ret), vg_name_exist);
       return LVM_EVGSPLIT_VG_READ_EXIST;
    }
 
@@ -250,9 +270,9 @@ int main ( int argc, char **argv) {
          fprintf ( stderr, "%s -- ERROR: can't split logical volumes; please "
                            "run pvmove\n\n", cmd);
       } else {
-         fprintf ( stderr, "%s -- ERROR %d splitting volume group \"%s\" "
+         fprintf ( stderr, "%s -- ERROR \"%s\" splitting volume group \"%s\" "
                            "into \"%s\"\n\n",
-                           cmd, ret, vg_name_exist, vg_name_new);
+                           cmd, lvm_error ( ret), vg_name_exist, vg_name_new);
       }
       return LVM_EVGSPLIT_VG_SETUP;
    }
@@ -279,9 +299,9 @@ int main ( int argc, char **argv) {
                                "of volume group \"%s\"\n",
                                cmd, lv_for_new[l], vg_name_exist);
       if ( ( ret = lv_release ( vg_exist_sav, lv_for_new[l])) != 0) {
-         fprintf ( stderr, "%s -- ERROR %d releasing logical volume \"%s\" "
+         fprintf ( stderr, "%s -- ERROR \"%s\" releasing logical volume \"%s\" "
                            "in VGDA of volume group \"%s\"\n",
-                           cmd, ret, lv_for_new[l], vg_name_exist);
+                           cmd, lvm_error ( ret), lv_for_new[l], vg_name_exist);
          continue;
       }
    }
@@ -298,9 +318,9 @@ int main ( int argc, char **argv) {
             fprintf ( stderr, "%s -- logical volume \"%s\" doesn't exist\n",
                               cmd, lv_for_new[l]);
          } else {
-            fprintf ( stderr, "%s -- ERROR %d getting status"
+            fprintf ( stderr, "%s -- ERROR \"%s\" getting status"
                               " of logical volume \"%s\"\n",
-                              cmd, ret, lv_for_new[l]);
+                              cmd, lvm_error ( ret), lv_for_new[l]);
          }
          return LVM_EVGSPLIT_LV_STATUS_BYNAME;
       }
@@ -310,9 +330,9 @@ int main ( int argc, char **argv) {
                                cmd, lv_for_new[l]);
       if ( opt_t == 0 &&
            ( ret = lv_remove ( vg_exist_sav, lv, lv_for_new[l])) < 0) {
-         fprintf ( stderr, "%s -- ERROR %d removing logical volume "
+         fprintf ( stderr, "%s -- ERROR \"%s\" removing logical volume "
                            "\"%s\" in kernel\n",
-                           cmd, ret, vg_exist_sav->lv[lr]->lv_name);
+                           cmd, lvm_error ( ret), vg_exist_sav->lv[lr]->lv_name);
          return LVM_EVGSPLIT_LV_REMOVE;
       }
    }
@@ -354,9 +374,10 @@ int main ( int argc, char **argv) {
                                pv[p]->pv_name);
       if ( opt_t == 0 &&
            ( ret = vg_reduce ( vg_name_exist, pv[p], vg_exist_sav)) < 0) {
-         fprintf ( stderr, "%s -- ERROR %d reducing volume group \"%s\" "
+         fprintf ( stderr, "%s -- ERROR \"%s\" reducing volume group \"%s\" "
                            "by physical volume \"%s\" in kernel\n",
-                           cmd, ret, vg_name_exist, pv[p]->pv_name);
+                           cmd, lvm_error ( ret),
+                           vg_name_exist, pv[p]->pv_name);
          return LVM_EVGSPLIT_VG_REDUCE;
       }
       free ( pv[p]);
@@ -368,8 +389,9 @@ int main ( int argc, char **argv) {
                             "\"%s\" on disk(s)\n", cmd, vg_name_exist);
    if ( opt_t == 0 &&
         ( ret = vg_write_with_pv_and_lv ( vg_exist)) < 0) {
-      fprintf ( stderr, "%s -- ERROR %d storing data of volume group "
-                        "\"%s\" on disk(s)\n\n", cmd, ret, vg_name_exist);
+      fprintf ( stderr, "%s -- ERROR \"%s\" storing data of volume group "
+                        "\"%s\" on disk(s)\n\n",
+                        cmd, lvm_error ( ret), vg_name_exist);
       return LVM_EVGSPLIT_VG_WRITE_EXIST;
    }
 
@@ -387,9 +409,9 @@ int main ( int argc, char **argv) {
                             "\"%s\" on disk(s)\n", cmd, vg_name_new);
    if ( opt_t == 0 &&
         ( ret = vg_write_with_pv_and_lv ( vg_new)) < 0) {
-      fprintf ( stderr, "%s -- ERROR %d storing data of volume group "
+      fprintf ( stderr, "%s -- ERROR \"%s\" storing data of volume group "
                         "\"%s\" on disk(s)\n\n",
-                        cmd, ret, vg_name_new);
+                        cmd, lvm_error ( ret), vg_name_new);
       return LVM_EVGSPLIT_VG_WRITE_NEW;
    }
 
@@ -401,6 +423,9 @@ int main ( int argc, char **argv) {
       printf ( "%s -- doing automatic backup of volume group \"%s\"\n",
                cmd, vg_name_new);
       vg_cfgbackup ( vg_name_new, VG_BACKUP_DIR, opt_v, vg_new);
+   } else {
+      printf ( "%s -- WARNING: you don't have an automatic backup of \"%s\"\n",
+               cmd, vg_name_new);
    }
 
 
@@ -414,9 +439,9 @@ int main ( int argc, char **argv) {
    if ( opt_t == 0 &&
         ( ret = vg_create_dir_and_group_and_nodes ( vg_exist,
                                                     opt_v)) < 0) {
-      fprintf ( stderr, "%s -- ERROR %d creating volume group "
+      fprintf ( stderr, "%s -- ERROR \"%s\" creating volume group "
                         "nodes of \"%s\"\n\n",
-                        cmd, ret, vg_name_exist);
+                        cmd, lvm_error ( ret), vg_name_exist);
       return LVM_EVGSPLIT_VG_CREATE_DIR_AND_GROUP_NODES_EXIST;
    }
 
@@ -426,9 +451,9 @@ int main ( int argc, char **argv) {
 
    if ( opt_t == 0 &&
         ( ret = vg_create_dir_and_group_and_nodes ( vg_new, opt_v)) < 0) {
-      fprintf ( stderr, "%s -- ERROR %d creating volume group "
+      fprintf ( stderr, "%s -- ERROR \"%s\" creating volume group "
                         "nodes of \"%s\"\n\n",
-                        cmd, ret, vg_name_new);
+                        cmd, lvm_error ( ret), vg_name_new);
       return LVM_EVGSPLIT_VG_CREATE_DIR_AND_GROUP_NODES_NEW;
    }
 
@@ -438,9 +463,9 @@ int main ( int argc, char **argv) {
                             cmd, vg_name_new);
    if ( opt_t == 0 &&
         ( ret = vg_create ( vg_name_new, vg_new)) < 0) {
-      fprintf ( stderr, "%s -- ERROR %d creating VGDA for volume group \"%s\" "
-                        "in kernel\n",
-                        cmd, ret, vg_name_new);
+      fprintf ( stderr, "%s -- ERROR \"%s\" creating VGDA for volume group "
+                        "\"%s\" in kernel\n",
+                        cmd, lvm_error ( ret), vg_name_new);
       if ( ret == -LVM_EVG_CREATE_REMOVE_OPEN)
          fprintf ( stderr, "%s -- LVM not in kernel?\n", cmd);
       fprintf ( stderr, "\n");
@@ -451,13 +476,13 @@ int main ( int argc, char **argv) {
                             cmd, vg_name_new);
    if ( opt_t == 0 &&
         ( ret = lvm_tab_vg_insert ( vg_name_new)) < 0) {
-      fprintf ( stderr, "%s -- ERROR %d inserting volume group \"%s\" "
+      fprintf ( stderr, "%s -- ERROR \"%s\" inserting volume group \"%s\" "
                         "into \"%s\"\n\n",
-                        cmd, ret, vg_name_new, LVMTAB);
+                        cmd, lvm_error ( ret), vg_name_new, LVMTAB);
       return LVM_EVGSPLIT_VG_INSERT;
    }
 
-   printf ( "%s -- volume group \"%s\" successfully splitted into "
+   printf ( "%s -- volume group \"%s\" successfully split into "
             "\"%s\" and \"%s\"\n",
             cmd, vg_name_exist, vg_name_exist, vg_name_new);
    if ( opt_t > 0) printf ( "%s -- this only has been a test run\n", cmd);
